@@ -24,7 +24,7 @@
 /*
  * I do not have much in the way of useful docs.
  * This is a guess based on some sample sheets with a few pointers from
- * 	http://www.vistasource.com/products/axware/fileformats/wptchc01.html
+ *	http://www.vistasource.com/products/axware/fileformats/wptchc01.html
  */
 
 #include <gnumeric-config.h>
@@ -55,8 +55,11 @@
 
 #include <goffice/app/io-context.h>
 #include <goffice/app/error-info.h>
+#include <goffice/utils/go-format.h>
 #include <goffice/utils/go-glib-extras.h>
+
 #include <gsf/gsf-input-textline.h>
+
 #include <string.h>
 #include <stdlib.h>
 
@@ -77,7 +80,7 @@ typedef struct {
 	GSList *sheet_order;
 	GSList *std_names, *real_names;
 
-	GnmExprConventions *exprconv;
+	GnmConventions *convs;
 } ApplixReadState;
 
 /* #define NO_DEBUG_APPLIX */
@@ -90,6 +93,22 @@ static int debug_applix_read = 0;
 
 #define a_strncmp(buf, str) strncmp ((buf), str, sizeof (str) - 1)
 
+static long
+au_strtol (const unsigned char *str, unsigned char **end)
+{
+	char *send;
+	long res = strtol ((const char *)str, &send, 10);
+	if (end) *end = (unsigned char *)send;
+	return res;
+}
+
+static long
+a_strtol (const char *str, char **end)
+{
+	return strtol (str, end, 10);
+}
+
+
 /* The maximum numer of character potentially involved in a new line */
 #define MAX_END_OF_LINE_SLOP	16
 
@@ -155,13 +174,13 @@ applix_parse_value (char *buf, char **follow)
 static gboolean
 valid_col (int c)
 {
-	return c >= 0 && c < SHEET_MAX_COLS;
+	return c >= 0 && c < gnm_sheet_get_max_cols (NULL);
 }
 
 static gboolean
 valid_row (int r)
 {
-	return r >= 0 && r < SHEET_MAX_ROWS;
+	return r >= 0 && r < gnm_sheet_get_max_rows (NULL);
 }
 
 static gboolean
@@ -235,7 +254,7 @@ applix_sheetref_parse (char const *start, Sheet **sheet, Workbook const *wb)
 
 static char const *
 applix_rangeref_parse (GnmRangeRef *res, char const *start, GnmParsePos const *pp,
-		       GnmExprConventions const *convention)
+		       GnmConventions const *convention)
 {
 	char const *ptr = start, *tmp1, *tmp2;
 	Workbook *wb = pp->wb;
@@ -332,6 +351,9 @@ applix_get_line (ApplixReadState *state)
 			break;
 	}
 
+	if (offset == 0 && ptr == NULL)
+		return NULL;
+
 	if (state->buffer != NULL)
 		state->buffer [offset] = '\0';
 	return state->buffer;
@@ -358,7 +380,7 @@ applix_read_colormap (ApplixReadState *state)
 			if (iter <= buffer || *iter != ' ')
 				return TRUE;
 
-			numbers[count] = strtol (iter+1, (char **)&end, 10);
+			numbers[count] = au_strtol (iter + 1, &end);
 			if (end != pos || numbers[count] < 0 || numbers[count] > 255)
 				return TRUE;
 		}
@@ -412,7 +434,7 @@ applix_get_color (ApplixReadState *state, char **buf)
 {
 	/* Skip 'FG' or 'BG' */
 	char *start = *buf+2;
-	int num = strtol (start, buf, 10);
+	int num = a_strtol (start, buf);
 
 	if (start == *buf) {
 		applix_parse_error (state, "Invalid color");
@@ -558,11 +580,12 @@ applix_parse_style (ApplixReadState *state, unsigned char **buffer)
 					}
 
 					if (!g_ascii_isdigit (sep[1]) ||
-					    (0 == (id = strtol (sep+1, &end, 10))) ||
+					    (0 == (id = a_strtol (sep+1, &end))) ||
 					    sep+1 == end ||
-					    id < 1 || id > 16)
+					    id < 1 || id > 16) {
 						applix_parse_error (state, "Unknown format %d", id);
-
+						return NULL;
+					}
 					format = date_formats[id - 1];
 					sep = end;
 					break;
@@ -724,9 +747,11 @@ applix_parse_style (ApplixReadState *state, unsigned char **buffer)
 					/* be a font ID numbered from 0 */
 					char *start = (sep += 2);
 
-					font_id = strtol (start, &sep, 10);
-					if (start == sep || font_id < 0 || font_id >= (int)state->font_names->len)
+					font_id = a_strtol (start, &sep);
+					if (start == sep || font_id < 0 || font_id >= (int)state->font_names->len) {
 						applix_parse_error (state, "Unknown font index %s", start);
+						font_id = 0;
+					}
 					break;
 				}
 
@@ -745,7 +770,8 @@ applix_parse_style (ApplixReadState *state, unsigned char **buffer)
 			return NULL;
 		}
 
-		gnm_style_set_font_name (style, g_ptr_array_index (state->font_names, font_id));
+		if (font_id < (int)state->font_names->len)
+			gnm_style_set_font_name (style, g_ptr_array_index (state->font_names, font_id));
 
 		/* Background, pattern, and borders */
 		for (++sep ; *sep && *sep != ')' ; ) {
@@ -760,11 +786,11 @@ applix_parse_style (ApplixReadState *state, unsigned char **buffer)
 					19, 20, 21, 22, 23,
 				};
 				char *end;
-				int num = strtol (sep += 2, &end, 10);
+				int num = a_strtol (sep += 2, &end);
 
 				if (sep == end || 0 >= num || num >= (int)G_N_ELEMENTS (map)) {
 					applix_parse_error (state, "Unknown pattern %s", sep);
-					return NULL;
+					goto error;
 				}
 
 				num = map[num];
@@ -774,14 +800,14 @@ applix_parse_style (ApplixReadState *state, unsigned char **buffer)
 				if (sep[0] == 'F' && sep[1] == 'G' ) {
 					GnmColor *color = applix_get_color (state, &sep);
 					if (color == NULL)
-						return NULL;
+						goto error;
 					gnm_style_set_pattern_color (style, color);
 				}
 
 				if (sep[0] == 'B' && sep[1] == 'G') {
 					GnmColor *color = applix_get_color (state, &sep);
 					if (color == NULL)
-						return NULL;
+						goto error;
 					gnm_style_set_back_color (style, color);
 				}
 			} else if (sep[0] == 'T' || sep[0] == 'B' || sep[0] == 'L' || sep[0] == 'R') {
@@ -802,18 +828,18 @@ applix_parse_style (ApplixReadState *state, unsigned char **buffer)
 				GnmStyleBorderOrientation const orient = (sep[0] == 'T' || sep[0] == 'B')
 					? GNM_STYLE_BORDER_HORIZONTAL : GNM_STYLE_BORDER_VERTICAL;
 				char *end;
-				int num = strtol (++sep, &end, 10);
+				int num = a_strtol (++sep, &end);
 
 				if (sep == end || 0 >= num || num >= (int)G_N_ELEMENTS (map)) {
 					applix_parse_error (state, "Unknown border style %s", sep);
-					return NULL;
+					goto error;
 				}
 				sep = end;
 
 				if (sep[0] == 'F' && sep[1] == 'G' ) {
 					color = applix_get_color (state, &sep);
 					if (color == NULL)
-						return NULL;
+						goto error;
 				} else
 					color = style_color_black ();
 
@@ -825,13 +851,13 @@ applix_parse_style (ApplixReadState *state, unsigned char **buffer)
 				++sep;
 			else if (*sep != ')') {
 				applix_parse_error (state, "Invalid pattern, background, or border");
-				return NULL;
+				goto error;
 			}
 		}
 
 		if (*sep != ')') {
 			applix_parse_error (state, "Invalid pattern or background");
-			return NULL;
+			goto error;
 		}
 
 		/* Store the newly parsed style along with its descriptor */
@@ -843,6 +869,11 @@ applix_parse_style (ApplixReadState *state, unsigned char **buffer)
 	*buffer = tmp + 2;
 	gnm_style_ref (style);
 	return style;
+
+error:
+	if (style)
+		gnm_style_unref (style);
+	return NULL;
 }
 
 static gboolean
@@ -987,7 +1018,7 @@ applix_read_view (ApplixReadState *state, unsigned char *buffer)
 						  &pos, pos.col, pos.row, pos.col, pos.row);
 		} else if (!a_strncmp (buffer, "View Default Column Width ")) {
 			char *ptr, *tmp = buffer + 26;
-			int width = strtol (tmp, &ptr, 10);
+			int width = a_strtol (tmp, &ptr);
 			if (tmp == ptr || width <= 0)
 				return applix_parse_error (state, "Invalid default column width");
 
@@ -995,7 +1026,7 @@ applix_read_view (ApplixReadState *state, unsigned char *buffer)
 				applix_width_to_pixels (width));
 		} else if (!a_strncmp (buffer, "View Default Row Height: ")) {
 			char *ptr, *tmp = buffer + 25;
-			int height = strtol (tmp, &ptr, 10);
+			int height = a_strtol (tmp, &ptr);
 			if (tmp == ptr || height <= 0)
 				return applix_parse_error (state, "Invalid default row height");
 
@@ -1008,10 +1039,10 @@ applix_read_view (ApplixReadState *state, unsigned char *buffer)
 				int row, height;
 				char *tmp;
 
-				row = strtol (tmp = ptr + 1, &ptr, 10) - 1;
+				row = a_strtol (tmp = ptr + 1, &ptr) - 1;
 				if (tmp == ptr || row < 0 || *ptr != ':')
 					return applix_parse_error (state, "Invalid row size row number");
-				height = strtol (tmp = ptr + 1, &ptr, 10);
+				height = a_strtol (tmp = ptr + 1, &ptr);
 				if (height >= 32768)
 					height -= 32768;
 
@@ -1037,7 +1068,7 @@ applix_read_view (ApplixReadState *state, unsigned char *buffer)
 				ptr = col_parse (tmp = ptr + 1, &col, &dummy);
 				if (!ptr || *ptr != ':')
 					return applix_parse_error (state, "Invalid column");
-				width = strtol (tmp = ptr + 1, (char **)&ptr, 10);
+				width = a_strtol (tmp = ptr + 1, (char **)&ptr);
 				if (tmp == ptr || width <= 0)
 					return applix_parse_error (state, "Invalid column size");
 
@@ -1150,7 +1181,7 @@ applix_read_cells (ApplixReadState *state)
 
 					if (!valid_cellpos (&r.start) || !valid_cellpos (&r.end)) {
 						g_warning ("Ignoring sheet contents beyond allowed range.");
-						continue;						
+						continue;
 					}
 
 					is_array = TRUE;
@@ -1169,7 +1200,7 @@ applix_read_cells (ApplixReadState *state)
 					texpr = gnm_expr_parse_str (expr_string+1,
 						parse_pos_init_cell (&pos, cell),
 								   GNM_EXPR_PARSE_DEFAULT,
-								   state->exprconv,
+								   state->convs,
 								   parse_error_init (&perr));
 
 				if (texpr == NULL) {
@@ -1257,7 +1288,7 @@ applix_read_row_list (ApplixReadState *state, unsigned char *ptr)
 	if (*ptr != '!')
 		return applix_parse_error (state, "Invalid row format");
 
-	r.start.row = r.end.row = strtol (++ptr, (char **)&tmp, 10) - 1;
+	r.start.row = r.end.row = au_strtol (++ptr, &tmp) - 1;
 	if (tmp == ptr || r.start.row < 0 || tmp[0] != ':' || tmp[1] != ' ')
 		return applix_parse_error (state, "Invalid row format row number");
 
@@ -1265,13 +1296,13 @@ applix_read_row_list (ApplixReadState *state, unsigned char *ptr)
 	do {
 		unsigned attr_index;
 
-		r.start.col = strtol (ptr = tmp+1, (char **)&tmp, 10);
+		r.start.col = au_strtol (ptr = tmp+1, &tmp);
 		if (tmp == ptr || r.start.col < 0 || tmp[0] != '-')
 			return applix_parse_error (state, "Invalid row format start col");
-		r.end.col = strtol (ptr = tmp+1, (char **)&tmp, 10);
+		r.end.col = au_strtol (ptr = tmp+1, &tmp);
 		if (tmp == ptr || r.end.col < 0 || tmp[0] != ':')
 			return applix_parse_error (state, "Invalid row format end col");
-		attr_index = strtol (ptr = tmp+1, (char **)&tmp, 10);
+		attr_index = au_strtol (ptr = tmp+1, &tmp);
 		if (tmp != ptr && attr_index >= 2 && attr_index < state->attrs->len+2) {
 			GnmStyle *style = g_ptr_array_index(state->attrs, attr_index-2);
 			gnm_style_ref (style);
@@ -1347,7 +1378,7 @@ applix_read_absolute_name (ApplixReadState *state, char *buffer)
 		return TRUE;
 	applix_rangeref_parse (&ref, end+2,
 		parse_pos_init (&pp, state->wb, NULL, 0, 0),
-		state->exprconv);
+		state->convs);
 	ref.a.col_relative = ref.b.col_relative =
 		ref.a.row_relative = ref.b.row_relative = FALSE;
 
@@ -1546,7 +1577,7 @@ cb_remove_style (gpointer key, gpointer value, gpointer user_data)
 }
 
 static GnmExpr const *
-applix_func_map_in (GnmExprConventions const *conv, Workbook *scope,
+applix_func_map_in (GnmConventions const *conv, Workbook *scope,
 		    char const *name, GnmExprList *args)
 {
 	static struct {
@@ -1565,7 +1596,7 @@ applix_func_map_in (GnmExprConventions const *conv, Workbook *scope,
 	int i;
 
 	if (NULL == namemap) {
-		namemap = g_hash_table_new (go_ascii_strcase_hash, 
+		namemap = g_hash_table_new (go_ascii_strcase_hash,
 					    go_ascii_strcase_equal);
 		for (i = 0; sc_func_renames[i].applix_name; i++)
 			g_hash_table_insert (namemap,
@@ -1581,10 +1612,10 @@ applix_func_map_in (GnmExprConventions const *conv, Workbook *scope,
 	return gnm_expr_new_funcall (f, args);
 }
 
-static GnmExprConventions *
-applix_conventions (void)
+static GnmConventions *
+applix_conventions_new (void)
 {
-	GnmExprConventions *conv = gnm_expr_conventions_new ();
+	GnmConventions *conv = gnm_conventions_new ();
 
 	conv->intersection_char		= 0;
 	conv->accept_hash_logicals	= TRUE;
@@ -1620,7 +1651,7 @@ applix_read (IOContext *io_context, WorkbookView *wb_view, GsfInput *src)
 	state.sheet_order = NULL;
 	state.std_names   = NULL;
 	state.real_names  = NULL;
-	state.exprconv    = applix_conventions ();
+	state.convs       = applix_conventions_new ();
 
 	/* Actually read the workbook */
 	res = applix_read_impl (&state);
@@ -1633,19 +1664,20 @@ applix_read (IOContext *io_context, WorkbookView *wb_view, GsfInput *src)
 	g_slist_free (state.sheet_order);
 
 	renamed_sheets = NULL;
-	for (ptr = state.std_names; ptr != NULL ; ptr = ptr->next)
+	for (ptr = state.std_names; ptr != NULL ; ptr = ptr->next) {
+		const char *name = ptr->data;
+		Sheet *sheet = workbook_sheet_by_name (state.wb, name);
+		int idx = sheet ? sheet->index_in_wb : -1;
 		renamed_sheets = g_slist_prepend (renamed_sheets,
-			GINT_TO_POINTER (workbook_sheet_by_name 
-					 (state.wb, ptr->data)->index_in_wb));
+						  GINT_TO_POINTER (idx));
+	}
 	renamed_sheets = g_slist_reverse (renamed_sheets);
 	workbook_sheet_rename (state.wb, renamed_sheets,
-			       state.real_names, 
+			       state.real_names,
 			       GO_CMD_CONTEXT (io_context));
 	g_slist_free (renamed_sheets);
-	g_slist_foreach (state.std_names, (GFunc)g_free, NULL);
-	g_slist_free (state.std_names);
-	g_slist_foreach (state.real_names, (GFunc)g_free, NULL);
-	g_slist_free (state.real_names);
+	go_slist_free_custom (state.std_names, g_free);
+	go_slist_free_custom (state.real_names, g_free);
 
 	/* Release the shared expressions and styles */
 	g_hash_table_foreach_remove (state.exprs, &cb_remove_texpr, NULL);
@@ -1668,5 +1700,5 @@ applix_read (IOContext *io_context, WorkbookView *wb_view, GsfInput *src)
 	if (state.parse_error != NULL)
 		gnumeric_io_error_info_set (io_context, state.parse_error);
 
-	gnm_expr_conventions_free (state.exprconv);
+	gnm_conventions_free (state.convs);
 }
