@@ -3,7 +3,7 @@
 /*
  * boot.c: the external interface to the MS Excel import/export
  *
- * Copyright (C) 2000-2006 Jody Goldberg (jody@gnome.org)
+ * Copyright (C) 2000-2007 Jody Goldberg (jody@gnome.org)
  * Copyright (C) 1998-2001 Michael Meeks (miguel@kernel.org)
  *
  * This program is free software; you can redistribute it and/or
@@ -41,6 +41,7 @@
 #include <gsf/gsf-input.h>
 #include <gsf/gsf-infile.h>
 #include <gsf/gsf-infile-msole.h>
+#include <gsf/gsf-infile-msvba.h>
 #include <gsf/gsf-msole-utils.h>
 #include <gsf/gsf-output-stdio.h>
 #include <gsf/gsf-outfile.h>
@@ -59,7 +60,7 @@ GNM_PLUGIN_MODULE_HEADER;
  * >1 increasing levels of detail.
  */
 gint ms_excel_read_debug = 0;
-gint ms_excel_pivot_debug = 10;
+gint ms_excel_pivot_debug = 0;
 gint ms_excel_escher_debug = 0;
 gint ms_excel_formula_debug = 0;
 gint ms_excel_chart_debug = 0;
@@ -138,15 +139,15 @@ excel_read_metadata (GsfDocMetaData *meta_data, GsfInfile *ole, char const *name
 	}
 }
 
-/**
- * excel_file_open
- * @fo:         File opener
- * @context:   	IO context
- * @wbv:    	Workbook view
- * @input:  	Input stream
- *
- * Load en excel workbook.
- **/
+#ifdef SPEW_VBA
+static void
+cb_dump_vba (char const *name, guint8 const *src_code)
+{
+	printf ("<module name=\"%s\">\n<![CDATA[%s]]>\n</module>\n", name, src_code);
+}
+#endif
+
+/* Service entry point */
 void
 excel_file_open (GOFileOpener const *fo, IOContext *context,
                  WorkbookView *wbv, GsfInput *input)
@@ -200,8 +201,24 @@ excel_file_open (GOFileOpener const *fo, IOContext *context,
 	/* See if there are any macros to keep around */
 	stream = gsf_infile_child_by_name (ole, "\01CompObj");
 	if (stream != NULL) {
-		GsfInput *macros = gsf_infile_child_by_name (ole, "_VBA_PROJECT_CUR");
+		GsfInput *macros = gsf_infile_child_by_vname (ole, "_VBA_PROJECT_CUR", "VBA", NULL);
 		if (macros != NULL) {
+			GsfInfile *vba = gsf_infile_msvba_new (GSF_INFILE (macros), NULL);
+			if (NULL != vba) {
+				GHashTable *modules =
+					gsf_infile_msvba_steal_modules (GSF_INFILE_MSVBA (vba));
+				if (NULL != modules) {
+#ifdef SPEW_VBA
+					g_hash_table_foreach (modules,
+						(GHFunc) cb_dump_vba, NULL);
+#endif
+					g_object_set_data_full (G_OBJECT (wb), "VBA",
+						modules, (GDestroyNotify) g_hash_table_destroy);
+				}
+				g_object_unref (G_OBJECT (vba));
+			}
+
+			/* LOOKS BROKEN */
 			g_object_set_data_full (G_OBJECT (wb), "MS_EXCEL_COMPOBJ",
 				gsf_structured_blob_read (stream), g_object_unref);
 			g_object_set_data_full (G_OBJECT (wb), "MS_EXCEL_MACROS",
@@ -319,7 +336,7 @@ go_plugin_init (GOPlugin *plugin, GOCmdContext *cc)
 	for (i = 0 ; i < excel_func_desc_size; i++) {
 		ExcelFuncDesc const *fd = excel_func_desc + i;
 		name = fd->name;
-		if (fd->flags & (XL_UNKNOWN | XL_MAGIC)) 
+		if (fd->flags & (XL_UNKNOWN | XL_MAGIC))
 			continue;
 		if (fd->flags & XL_XLM) {
 			if (fd->flags != XL_XLM)
