@@ -22,9 +22,13 @@
 #ifndef GNUMERIC_PLUGIN_LATEX_H
 #define GNUMERIC_PLUGIN_LATEX_H
 
-#include "gnumeric.h"
+#include <gnumeric.h>
+#include <goffice/app/file.h>
 
-void latex_file_save (GnumFileSaver const *fs, IOContext *io_context, WorkbookView *wb_view, const gchar *file_name);
-void latex2e_file_save (GnumFileSaver const *fs, IOContext *io_context, WorkbookView *wb_view, const gchar *file_name);
+void latex_file_save (GOFileSaver const *fs, IOContext *io_context,
+		      WorkbookView const *wb_view, GsfOutput *output);
+
+void latex_table_file_save (GOFileSaver const *fs, IOContext *io_context,
+		            WorkbookView const *wb_view, GsfOutput *output);
 
 #endif
