@@ -47,14 +47,25 @@ typedef struct {
 } GnmPerlPluginLoader;
 typedef GObjectClass GnmPerlPluginLoaderClass;
 
+static const char help_template_text[] =
+	N_("@FUNCTION=PERL_FUNCTION_TEMPLATE\n"
+	   "@SYNTAX=PERL_FUNCTION_TEMPLATE(value1, value2, ...)\n"
+	   "@DESCRIPTION="
+	   "This is the perl function template. \n");
+
+static GnmFuncHelp help_template[] = {
+	{ GNM_FUNC_HELP_OLD, help_template_text },
+	{ GNM_FUNC_HELP_END }
+};
+
 static GnmValue*
 call_perl_function_args (GnmFuncEvalInfo *ei, GnmValue const * const *args)
 {
 	GnmFunc const *fndef;
 	gint min_n_args, max_n_args, n_args;
 	gint i;
-	gchar *perl_func = NULL;
-	GnmValue* result = NULL;
+	gchar *perl_func;
+	GnmValue* result;
 	dSP;
 
 	fndef = ei->func_call->func;
@@ -66,10 +77,9 @@ call_perl_function_args (GnmFuncEvalInfo *ei, GnmValue const * const *args)
 	ENTER;
 	SAVETMPS;
 	PUSHMARK(SP);
-	for (i=0;i<n_args;i++) {
-		gnm_float f = value_get_as_float (args[i]);
-		int i = (int)i;  /* FIXME: someone needs to figure out what this is.  */
-		XPUSHs(sv_2mortal(newSViv(i)));
+	for (i = 0; i < n_args; i++) {
+		SV* sv = value2perl (args[i]);
+		XPUSHs(sv_2mortal(sv));
 	}
 	PUTBACK;
 	call_pv (perl_func, G_EVAL | G_SCALAR);
@@ -154,10 +164,10 @@ gplp_func_desc_load (GOPluginService *service,
 	res->name = g_strdup(name);
 	res->arg_spec = arg_spec;
 	res->arg_names = arg_names;
-#warning FIXME adapt for the new GnmFuncHelp struct.
-#if 0
-	res->help = (const char**)&help_text;
-#endif
+
+	help_template[0].text = help_text ? help_text : help_template_text;
+
+	res->help = g_slice_dup (GnmFuncHelp, help_template);
 	res->fn_args = NULL;
 	res->fn_args = &call_perl_function_args;
 	res->fn_nodes = NULL;
@@ -191,12 +201,15 @@ gplp_load_base (GOPluginLoader *loader, ErrorInfo **ret_error)
 {
 	char *argv[] = { (char*)"", NULL, NULL, NULL };
 	char const *arg;
+	int argc;
 
 	arg = go_plugin_get_dir_name (go_plugin_loader_get_plugin (loader));
 	argv[1] = g_strconcat ("-I", arg, NULL);
 	argv[2] = g_build_filename (arg, "perl_func.pl", NULL);
+	argc = 2;
 
 	if (g_file_test (argv[2], G_FILE_TEST_EXISTS)) {
+		PERL_SYS_INIT3(&argc, &argv, NULL);
 		gnm_perl_interp = perl_alloc ();
 		perl_construct (gnm_perl_interp);
 		perl_parse (gnm_perl_interp, xs_init, 3, argv, NULL);
