@@ -8,7 +8,7 @@
 
 #define GNM_PY_COMMAND_LINE_TYPE     (gnm_py_command_line_get_type ())
 #define GNM_PY_COMMAND_LINE(o)       (G_TYPE_CHECK_INSTANCE_CAST ((o), GNM_PY_COMMAND_LINE_TYPE, GnmPyCommandLine))
-#define GNM_IS_PY_COMMAND_LINE(o)    (G_TYPE_CHECK_INSTANCE_TYPE ((o), GNM_PY_COMMAND_LINE_TYPE))
+#define IS_GNM_PY_COMMAND_LINE(o)    (G_TYPE_CHECK_INSTANCE_TYPE ((o), GNM_PY_COMMAND_LINE_TYPE))
 
 GType gnm_py_command_line_get_type (void);
 typedef struct _GnmPyCommandLine GnmPyCommandLine;
