@@ -6,10 +6,7 @@
  */
 
 #include <gnumeric-config.h>
-#include <Python.h>
 #include <gnumeric.h>
-#include <glib.h>
-#include "pygobject.h"
 #include "application.h"
 #include "workbook.h"
 #include "cell.h"
@@ -22,13 +19,18 @@
 #include "func.h"
 #include "str.h"
 #include "wbc-gtk.h"
-#include <goffice/app/go-plugin.h>
 #include "parse-util.h"
 #include "gnm-py-interpreter.h"
 #include "py-gnumeric.h"
 
+#include <goffice/app/go-plugin.h>
+
 #include <glib/gi18n-lib.h>
 
+#include <Python.h>
+#define NO_IMPORT_PYGOBJECT
+#include <pygobject.h>
+
 static PyTypeObject py_Boolean_object_type;
 typedef struct _py_Boolean_object py_Boolean_object;
 static PyObject *py_new_Boolean_object (gboolean value);
@@ -256,7 +258,7 @@ py_obj_to_gnm_value (const GnmEvalPos *eval_pos, PyObject *py_val)
 
 		py_val_type_str = PyObject_Str (py_val_type);
 		msg = g_strdup_printf (_("Unsupported Python type: %s"),
-		                       PyString_AsString (py_val_type_str));
+				       PyString_AsString (py_val_type_str));
 		ret_val = value_new_error (eval_pos, msg);
 		g_free (msg);
 		Py_DECREF (py_val_type_str);
@@ -289,11 +291,11 @@ py_exc_to_string (void)
 		if (exc_value != NULL) {
 			exc_value_str = PyObject_Str (exc_value);
 			error_str = g_strdup_printf (_("Python exception (%s: %s)"),
-			                             PyString_AsString (exc_type_str),
-			                             PyString_AsString (exc_value_str));
+						     PyString_AsString (exc_type_str),
+						     PyString_AsString (exc_value_str));
 		} else {
 			error_str = g_strdup_printf (_("Python exception (%s)"),
-			                             PyString_AsString (exc_type_str));
+						     PyString_AsString (exc_type_str));
 		}
 	}
 
@@ -328,7 +330,7 @@ gnm_value_to_py_obj (const GnmEvalPos *eval_pos, const GnmValue *val)
 	case VALUE_BOOLEAN:
 		py_val = py_new_Boolean_object (val->v_bool.val);
 		break;
-    	case VALUE_FLOAT:
+	case VALUE_FLOAT:
 		py_val = PyFloat_FromDouble (value_get_as_float (val));
 		break;
 	case VALUE_STRING:
@@ -394,7 +396,7 @@ python_call_gnumeric_function (GnmFunc *fn_def, const GnmEvalPos *opt_eval_pos,
 	}
 	if (eval_pos == NULL) {
 		PyErr_SetString (GNUMERIC_MODULE_GET ("GnumericError"),
-		                 "Missing Evaluation Position.");
+				 "Missing Evaluation Position.");
 		return NULL;
 	}
 
@@ -434,8 +436,8 @@ call_python_function (PyObject *python_fn, GnmEvalPos const *eval_pos, gint n_ar
 	python_args = PyTuple_New (n_args);
 	g_return_val_if_fail (python_args != NULL, NULL);
 	for (i = 0; i < n_args; i++) {
-		(void) PyTuple_SetItem (python_args, i, 
-					gnm_value_to_py_obj (eval_pos, 
+		(void) PyTuple_SetItem (python_args, i,
+					gnm_value_to_py_obj (eval_pos,
 							     args[i]));
 	}
 	if (get_eval_pos () != NULL) {
@@ -581,7 +583,7 @@ py_CellPos_object_dealloc (py_CellPos_object *self)
 static PyObject *
 py_CellPos_object_str (py_CellPos_object *self)
 {
-	return PyString_FromString 
+	return PyString_FromString
 		((char *) cellpos_as_string (&self->cell_pos));
 }
 
@@ -667,8 +669,8 @@ py_Range_get_tuple_method (py_Range_object *self, PyObject *args)
 	}
 
 	return Py_BuildValue ((char *) "(iiii)",
-	                      self->range.start.col, self->range.start.row,
-	                      self->range.end.col, self->range.end.row);
+			      self->range.start.col, self->range.start.row,
+			      self->range.end.col, self->range.end.row);
 }
 
 static PyObject *
@@ -744,7 +746,7 @@ static PyTypeObject py_Range_object_type = {
  */
 
 /*
- * FIXME: The sheet and cells the GnmCellRef refer to may be deleted 
+ * FIXME: The sheet and cells the GnmCellRef refer to may be deleted
  * behind our backs.
  */
 
@@ -781,13 +783,13 @@ py_CellRef_object_getattr (py_CellRef_object *self, gchar *name)
 			result = Py_None;
 		}
 	} else if (strcmp (name, "col_relative") == 0) {
-		result = Py_BuildValue ((char *) "i", 
+		result = Py_BuildValue ((char *) "i",
 				      self->cell_ref.col_relative ? 1 : 0);
 	} else if (strcmp (name, "row_relative") == 0) {
-		result = Py_BuildValue ((char *) "i", 
+		result = Py_BuildValue ((char *) "i",
 				      self->cell_ref.row_relative ? 1 : 0);
 	} else {
-		result = Py_FindMethod (py_CellRef_object_methods, 
+		result = Py_FindMethod (py_CellRef_object_methods,
 				      (PyObject *) self, name);
 	}
 
@@ -844,7 +846,7 @@ static PyTypeObject py_CellRef_object_type = {
  */
 
 /*
- * FIXME: The sheet and cells the GnmRangeRef refer to may be deleted 
+ * FIXME: The sheet and cells the GnmRangeRef refer to may be deleted
  * behind our backs.
  */
 
@@ -857,7 +859,7 @@ static PyObject *
 py_RangeRef_get_tuple_method (py_RangeRef_object *self, PyObject *args);
 
 static struct PyMethodDef py_RangeRef_object_methods[] = {
-	{(char *) "get_tuple", (PyCFunction) py_RangeRef_get_tuple_method,   
+	{(char *) "get_tuple", (PyCFunction) py_RangeRef_get_tuple_method,
 	 METH_VARARGS},
 	{NULL, NULL}
 };
@@ -875,7 +877,7 @@ py_RangeRef_get_tuple_method (py_RangeRef_object *self, PyObject *args)
 		return NULL;
 	}
 
-	return Py_BuildValue ((char *) "(O&O&)", 
+	return Py_BuildValue ((char *) "(O&O&)",
 			      py_new_CellRef_object, &self->range_ref.a,
 			      py_new_CellRef_object, &self->range_ref.b);
 }
@@ -1474,7 +1476,7 @@ py_sheet_style_get_method (py_Sheet_object *self, PyObject *args)
 	} else {
 		PyErr_Clear ();
 		if (!PyArg_ParseTuple (args, (char *) "O!:style_get",
-		                      &py_CellPos_object_type, &py_cell_pos)) {
+				      &py_CellPos_object_type, &py_cell_pos)) {
 			return NULL;
 		}
 		c = py_cell_pos->cell_pos.col;
@@ -1491,8 +1493,8 @@ py_sheet_style_apply_range_method (py_Sheet_object *self, PyObject *args)
 	py_Style_object *py_style;
 
 	if (!PyArg_ParseTuple (args, (char *) "O!O!:style_apply_range",
-	                       &py_Range_object_type, &py_range,
-	                       &py_Style_object_type, &py_style)) {
+			       &py_Range_object_type, &py_range,
+			       &py_Style_object_type, &py_style)) {
 		return NULL;
 	}
 
@@ -1510,8 +1512,8 @@ py_sheet_style_set_range_method (py_Sheet_object *self, PyObject *args)
 	py_Style_object *py_style;
 
 	if (!PyArg_ParseTuple (args, (char *) "O!O!:style_set_range",
-	                       &py_Range_object_type, &py_range,
-	                       &py_Style_object_type, &py_style)) {
+			       &py_Range_object_type, &py_range,
+			       &py_Style_object_type, &py_style)) {
 		return NULL;
 	}
 
@@ -1530,13 +1532,13 @@ py_sheet_style_set_pos_method (py_Sheet_object *self, PyObject *args)
 	py_Style_object *py_style;
 
 	if (PyArg_ParseTuple (args, (char *) "iiO!:style_set_pos",
-	                      &col, &row, &py_Style_object_type, &py_style)) {
+			      &col, &row, &py_Style_object_type, &py_style)) {
 		;
 	} else {
 		PyErr_Clear ();
 		if (!PyArg_ParseTuple (args, (char *) "O!O!:style_set_pos",
-		                       &py_CellPos_object_type, &py_cell_pos,
-		                       &py_Style_object_type, &py_style)) {
+				       &py_CellPos_object_type, &py_cell_pos,
+				       &py_Style_object_type, &py_style)) {
 			return NULL;
 		}
 	}
@@ -1731,7 +1733,7 @@ py_Workbook_gui_add (py_Workbook_object *self, PyObject *args)
 
 	if (workbook_sheet_count (self->wb) == 0)
 		(void)workbook_sheet_add (self->wb, -1);
-	
+
 	wbcg = wbc_gtk_new (NULL, self->wb, NULL, NULL);
 	result = py_new_Gui_object (wbcg);
 	g_object_unref (wbcg);    /* py_new_Gui_object added a reference */
@@ -1819,7 +1821,7 @@ py_Gui_get_workbook (py_Gui_object *self, PyObject *args)
 	if (!PyArg_ParseTuple (args, (char *) ":get_workbook")) {
 		return NULL;
 	}
-	
+
 	workbook = wb_control_get_workbook (WORKBOOK_CONTROL (self->wbcg));
 
 	return py_new_Workbook_object (workbook);
@@ -1833,11 +1835,11 @@ py_Gui_get_window (py_Gui_object *self, PyObject *args)
 	if (!PyArg_ParseTuple (args, (char *) ":get_window")) {
 		return NULL;
 	}
-	
+
 	g_return_val_if_fail (_PyGObject_API != NULL, NULL);
 
 	toplevel = wbcg_toplevel (self->wbcg);
-	
+
 	return pygobject_new (G_OBJECT(toplevel));
 }
 
@@ -2228,7 +2230,7 @@ py_gnumeric_Range_method (PyObject *self, PyObject *args)
 	py_CellPos_object *py_start, *py_end;
 
 	if (PyArg_ParseTuple (args, (char *) "iiii:Range",
-	                      &start_col, &start_row, &end_col, &end_row)) {
+			      &start_col, &start_row, &end_col, &end_row)) {
 		GnmCellPos start, end;
 		start.col = start_col; start.row = start_row;
 		end.col = end_col; end.row = end_row;
@@ -2236,10 +2238,10 @@ py_gnumeric_Range_method (PyObject *self, PyObject *args)
 	} else {
 		PyErr_Clear ();
 		if (PyArg_ParseTuple (args, (char *) "O!O!:Range",
-		                      &py_CellPos_object_type, &py_start,
-		                      &py_CellPos_object_type, &py_end)) {
+				      &py_CellPos_object_type, &py_start,
+				      &py_CellPos_object_type, &py_end)) {
 			result = py_new_Range_object_from_start_end (&py_start->cell_pos,
-			                                             &py_end->cell_pos);
+								     &py_end->cell_pos);
 		} else {
 			return NULL;
 		}
@@ -2298,7 +2300,7 @@ static PyMethodDef GnumericMethods[] = {
 	{ (char *) "CellPos", py_gnumeric_CellPos_method, METH_VARARGS },
 	{ (char *) "Range",   py_gnumeric_Range_method,   METH_VARARGS },
 	{ (char *) "GnmStyle",  py_gnumeric_Style_method,  METH_VARARGS },
- 	{ (char *) "workbooks", py_gnumeric_workbooks_method, METH_VARARGS },
+	{ (char *) "workbooks", py_gnumeric_workbooks_method, METH_VARARGS },
 	{ (char *) "workbook_new", py_gnumeric_workbook_new,  METH_VARARGS},
 	{ NULL, NULL },
 };
@@ -2368,6 +2370,6 @@ py_initgnumeric (GnmPyInterpreter *interpreter)
 		py_pinfo = Py_None;
 		Py_INCREF (Py_None);
 	}
-	(void) PyDict_SetItemString (module_dict, 
+	(void) PyDict_SetItemString (module_dict,
 				     (char *) "plugin_info", py_pinfo);
 }
