@@ -1,3 +1,4 @@
+/* vim: set sw=8: -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * auto-format.c: Suggest formats for expressions.
  *
@@ -59,20 +60,18 @@
 #define AF_EXPLICIT ((GnmFuncFlags)(GNM_FUNC_AUTO_MASK + 1))
 
 static GnmFuncFlags do_af_suggest_list (int argc,
-					const GnmExprConstPtr *argv,
+					GnmExprConstPtr const *argv,
 					GnmEvalPos const *epos,
 					GOFormat **explicit);
 
 struct cb_af_suggest { GnmFuncFlags typ; GOFormat **explicit; };
 
 static GnmValue *
-cb_af_suggest (G_GNUC_UNUSED Sheet *sheet,
-	       G_GNUC_UNUSED int col, G_GNUC_UNUSED int row,
-	       GnmCell *cell, void *_data)
+cb_af_suggest (GnmCellIter const *iter, gpointer user)
 {
-	struct cb_af_suggest *data = _data;
+	struct cb_af_suggest *data = user;
 
-	*(data->explicit) = cell_get_format (cell);
+	*(data->explicit) = gnm_cell_get_format (iter->cell);
 	if (*(data->explicit)) {
 		data->typ = AF_EXPLICIT;
 		return VALUE_TERMINATE;
@@ -84,11 +83,11 @@ static gboolean
 is_date (GnmFuncFlags typ, GOFormat *explicit)
 {
 	return (typ == GNM_FUNC_AUTO_DATE ||
-		(typ == AF_EXPLICIT && explicit->family == GO_FORMAT_DATE));
+		(typ == AF_EXPLICIT && go_format_is_date (explicit)));
 }
 
 static GnmFuncFlags
-do_af_suggest (GnmExpr const *expr, const GnmEvalPos *epos, GOFormat **explicit)
+do_af_suggest (GnmExpr const *expr, GnmEvalPos const *epos, GOFormat **explicit)
 {
 #if 0
 	{
@@ -99,7 +98,7 @@ do_af_suggest (GnmExpr const *expr, const GnmEvalPos *epos, GOFormat **explicit)
 		g_printerr ("Suggesting format for [%s]\n",
 			    gnm_expr_as_string (expr,
 						&pp,
-						gnm_expr_conventions_default));
+						gnm_conventions_default));
 	}
 #endif
 
@@ -212,10 +211,11 @@ do_af_suggest (GnmExpr const *expr, const GnmEvalPos *epos, GOFormat **explicit)
 		if (cell == NULL)
 			return GNM_FUNC_AUTO_UNKNOWN;
 
-		*explicit = cell_get_format (cell);
+		*explicit = gnm_cell_get_format (cell);
 		return *explicit ? AF_EXPLICIT : GNM_FUNC_AUTO_UNKNOWN;
 	}
 
+	case GNM_EXPR_OP_PAREN:
 	case GNM_EXPR_OP_UNARY_NEG:
 	case GNM_EXPR_OP_UNARY_PLUS:
 		return do_af_suggest (expr->unary.value, epos, explicit);
@@ -234,8 +234,8 @@ do_af_suggest (GnmExpr const *expr, const GnmEvalPos *epos, GOFormat **explicit)
 }
 
 static GnmFuncFlags
-do_af_suggest_list (int argc, const GnmExprConstPtr *argv,
-		    const GnmEvalPos *epos, GOFormat **explicit)
+do_af_suggest_list (int argc, GnmExprConstPtr const *argv,
+		    GnmEvalPos const *epos, GOFormat **explicit)
 {
 	int i;
 
