@@ -56,7 +56,7 @@ sv_select_cur_row (SheetView *sv)
 		sv_selection_reset (sv);
 		sv_selection_add_full (sv,
 			sv->edit_pos.col, sv->edit_pos.row,
-			0, r.start.row, SHEET_MAX_COLS-1, r.end.row);
+			0, r.start.row, gnm_sheet_get_max_cols (sv->sheet)-1, r.end.row);
 		sheet_update (sv->sheet);
 	}
 }
@@ -76,7 +76,7 @@ sv_select_cur_col (SheetView *sv)
 		sv_selection_reset (sv);
 		sv_selection_add_full (sv,
 			sv->edit_pos.col, sv->edit_pos.row,
-			r.start.col, 0, r.end.col, SHEET_MAX_ROWS-1);
+			r.start.col, 0, r.end.col, gnm_sheet_get_max_rows (sv->sheet)-1);
 		sheet_update (sv->sheet);
 	}
 }
@@ -373,7 +373,7 @@ cmd_paste_to_selection (WorkbookControl *wbc, SheetView *dest_sv, int paste_flag
 
 /**
  * cmd_shift_rows:
- * @wbc : 	The error context.
+ * @wbc :	The error context.
  * @sheet	the sheet
  * @col		column marking the start of the shift
  * @start_row	first row
@@ -399,7 +399,7 @@ cmd_shift_rows (WorkbookControl *wbc, Sheet *sheet,
 	rinfo.origin.start.row = start_row;
 	rinfo.origin.start.col = col;
 	rinfo.origin.end.row = end_row;
-	rinfo.origin.end.col = SHEET_MAX_COLS-1;
+	rinfo.origin.end.col = gnm_sheet_get_max_cols (sheet)-1;
 	if (count > 0)
 		rinfo.origin.end.col -= count;
 
@@ -412,7 +412,7 @@ cmd_shift_rows (WorkbookControl *wbc, Sheet *sheet,
 
 /**
  * cmd_shift_cols:
- * @wbc: 	The error context.
+ * @wbc:	The error context.
  * @sheet:	the sheet
  * @start_col:	first column
  * @end_col:	end column
@@ -438,7 +438,7 @@ cmd_shift_cols (WorkbookControl *wbc, Sheet *sheet,
 	rinfo.origin.start.col = start_col;
 	rinfo.origin.start.row = row;
 	rinfo.origin.end.col = end_col;
-	rinfo.origin.end.row = SHEET_MAX_ROWS-1;
+	rinfo.origin.end.row = gnm_sheet_get_max_rows (sheet)-1;
 	if (count > 0)
 		rinfo.origin.end.row -= count;
 
