@@ -64,7 +64,7 @@ typedef struct {
 	GtkTreeView   *treeview;
 	GtkListStore  *model_f;
 	GtkTreeView   *treeview_f;
-	GtkTextBuffer   *description;
+	GtkTextView   *description_view;
 
 	GSList *recent_funcs;
 
@@ -394,6 +394,14 @@ describe_new_style (GtkTextBuffer *description, GnmFunc const *func)
 			ADD_TEXT ("\n");
 			break;
 		}
+		case GNM_FUNC_HELP_NOTE: {
+			const char *text = F_(help->text);
+			ADD_TEXT ("\n");
+			ADD_TEXT (_("Note: "));
+			ADD_TEXT (text);
+			ADD_TEXT ("\n");
+			break;
+		}
 		case GNM_FUNC_HELP_SEEALSO: {
 			const char *text = help->text;  /* Not translated */
 
@@ -410,6 +418,7 @@ describe_new_style (GtkTextBuffer *description, GnmFunc const *func)
 				text = *end ? end + 1 : end;
 			}
 			ADD_TEXT ("\n");
+			break;
 		}
 		case GNM_FUNC_HELP_END:
 			return;
@@ -430,8 +439,15 @@ cb_dialog_function_select_fun_selection_changed (GtkTreeSelection *the_selection
 	GtkTreeIter  iter;
 	GtkTreeModel *model;
 	GnmFunc const *func;
+	GtkTextBuffer *description;
+	GtkTextMark *mark;
 
-	gtk_text_buffer_set_text (state->description, "", 0);
+	description =  gtk_text_view_get_buffer (state->description_view);
+
+	mark = gtk_text_buffer_get_mark (description, "start-mark");
+	gtk_text_view_scroll_to_mark (state->description_view, mark,
+				      0.1, TRUE, 0.0, 0.0);
+	gtk_text_buffer_set_text (description, "", 0);
 
 	if (gtk_tree_selection_get_selected (the_selection, &model, &iter)) {
 		gtk_tree_model_get (model, &iter,
@@ -441,11 +457,12 @@ cb_dialog_function_select_fun_selection_changed (GtkTreeSelection *the_selection
 		gnm_func_load_if_stub ((GnmFunc *)func);
 
 		if (func->help == NULL)
-			gtk_text_buffer_set_text (state->description, "?", -1);
+			gtk_text_buffer_set_text (description, "?", -1);
 		else if (func->help[0].type == GNM_FUNC_HELP_OLD)
-			describe_old_style (state->description, func);
+			describe_old_style (description, func);
 		else
-			describe_new_style (state->description, func);
+			describe_new_style (description, func);
+
 		gtk_widget_set_sensitive (state->ok_button, TRUE);
 	} else {
 		gtk_widget_set_sensitive (state->ok_button, FALSE);
@@ -459,7 +476,7 @@ cb_dialog_function_select_cat_selection_changed (GtkTreeSelection *the_selection
 	GtkTreeIter  iter;
 	GtkTreeModel *model;
 	GnmFuncGroup const * cat;
-	GSList *funcs, *ptr;
+	GSList *funcs = NULL, *ptr;
 	GnmFunc const *func;
 
 	gtk_list_store_clear (state->model_f);
@@ -470,8 +487,8 @@ cb_dialog_function_select_cat_selection_changed (GtkTreeSelection *the_selection
 				    -1);
 		if (cat != NULL) {
 			if (cat == GINT_TO_POINTER(-1)) {
+				/*  Show all functions */
 				int i = 0;
-				funcs = NULL;
 
 				while ((cat = gnm_func_group_get_nth (i++)) != NULL)
 					funcs = g_slist_concat (funcs,
@@ -479,39 +496,30 @@ cb_dialog_function_select_cat_selection_changed (GtkTreeSelection *the_selection
 
 				funcs = g_slist_sort (funcs,
 						      dialog_function_select_by_name);
-			} else
+			} else 
+				/* Show category cat */
 				funcs = g_slist_sort (g_slist_copy (cat->functions),
 						      dialog_function_select_by_name);
 
-			for (ptr = funcs; ptr; ptr = ptr->next) {
-				func = ptr->data;
-				if (!(func->flags & GNM_FUNC_INTERNAL)) {
-					gtk_list_store_append (state->model_f, &iter);
-					gtk_list_store_set (state->model_f, &iter,
-						FUN_NAME, gnm_func_get_name (func),
-						FUNCTION, func,
-						-1);
-				}
-			}
-			g_slist_free (funcs);
-		} else if (cat == NULL) {
-			for (ptr = state->recent_funcs; ptr != NULL; ptr = ptr->next) {
-				func = ptr->data;
+		} else
+			/* Show recent functions */
+			funcs = state->recent_funcs;
+
+		for (ptr = funcs; ptr; ptr = ptr->next) {
+			func = ptr->data;
+			if (!(func->flags & GNM_FUNC_INTERNAL)) {
 				gtk_list_store_append (state->model_f, &iter);
 				gtk_list_store_set (state->model_f, &iter,
-					FUN_NAME, gnm_func_get_name (func),
-					FUNCTION, func,
-					-1);
+						    FUN_NAME, gnm_func_get_name (func),
+						    FUNCTION, func,
+						    -1);
 			}
-		} else {
-			int i = 0;
-			funcs = NULL;
-
-			while ((cat = gnm_func_group_get_nth (i++)) != NULL)
-				funcs = g_slist_concat (funcs, g_slist_copy (cat->functions));
-
-			funcs = g_slist_sort (funcs, dialog_function_select_by_name);
 		}
+
+		gtk_tree_view_scroll_to_point (state->treeview_f, 0, 0);
+
+		if (funcs != state->recent_funcs)
+			g_slist_free (funcs);
 	}
 }
 
@@ -521,7 +529,8 @@ dialog_function_select_init (FunctionSelectState *state)
 	GtkWidget *scrolled;
 	GtkTreeViewColumn *column;
 	GtkTreeSelection *selection;
-	GtkTextView *textview;
+	GtkTextIter where;
+	GtkTextBuffer *description;
 
 	dialog_function_load_recent_funcs (state);
 
@@ -574,8 +583,10 @@ dialog_function_select_init (FunctionSelectState *state)
 	gtk_paned_set_position (GTK_PANED (glade_xml_get_widget
 					   (state->gui, "vpaned1")), 300);
 
-	textview = GTK_TEXT_VIEW (glade_xml_get_widget (state->gui, "description"));
-	state->description =  gtk_text_view_get_buffer (textview);
+	state->description_view = GTK_TEXT_VIEW (glade_xml_get_widget (state->gui, "description"));
+	description = gtk_text_view_get_buffer (state->description_view);
+	gtk_text_buffer_get_start_iter (description, &where);
+	gtk_text_buffer_create_mark (description, "start-mark", &where, TRUE);
 
 	state->ok_button = glade_xml_get_widget (state->gui, "ok_button");
 	gtk_widget_set_sensitive (state->ok_button, FALSE);
