@@ -31,6 +31,7 @@
 #include "dialogs.h"
 #include "help.h"
 #include <application.h>
+#include <wbc-gtk.h>
 #include <gnumeric-gconf.h>
 #include <gutils.h>
 #include <gui-util.h>
@@ -271,8 +272,7 @@ pm_gui_load_directory_page (PluginManagerGUI *pm_gui)
 	if (plugin_path_env != NULL) {
 		plugin_dirs = go_strsplit_to_slist (plugin_path_env, G_SEARCHPATH_SEPARATOR);
 		pm_gui_load_directories (pm_gui, plugin_dirs, FALSE);
-		g_slist_foreach (plugin_dirs, (GFunc)g_free, NULL);
-		g_slist_free (plugin_dirs);
+		go_slist_free_custom (plugin_dirs, g_free);
 	}
 	pm_gui_load_directories (pm_gui, gnm_app_prefs->plugin_extra_dirs, TRUE);
 }
@@ -458,8 +458,7 @@ cb_pm_selection_changed (GtkTreeSelection *selection, PluginManagerGUI *pm_gui)
 }
 
 static void
-pm_dialog_cleanup (G_GNUC_UNUSED GObject *dialog,
-		   PluginManagerGUI *pm_gui)
+cb_pm_dialog_free (PluginManagerGUI *pm_gui)
 {
 	GtkTreeModel *model = GTK_TREE_MODEL (pm_gui->model_plugins);
 	GtkTreeIter iter;
@@ -478,12 +477,14 @@ pm_dialog_cleanup (G_GNUC_UNUSED GObject *dialog,
 			G_OBJECT (plugin), (GWeakNotify) cb_plugin_destroyed, pm_gui);
 	}
 
-	if (pm_gui->gui != NULL) {
+	if (pm_gui->gui != NULL)
 		g_object_unref (G_OBJECT (pm_gui->gui));
-		pm_gui->gui = NULL;
-	}
-
-	pm_gui->dialog_pm = NULL;
+	if (pm_gui->model_plugins != NULL)
+		g_object_unref (G_OBJECT (pm_gui->model_plugins));
+	if (pm_gui->model_details != NULL)
+		g_object_unref (G_OBJECT (pm_gui->model_details));
+	if (pm_gui->model_directories != NULL)
+		g_object_unref (G_OBJECT (pm_gui->model_directories));
 	g_free (pm_gui);
 }
 
@@ -525,9 +526,6 @@ pm_dialog_init (PluginManagerGUI *pm_gui)
 	g_signal_connect (G_OBJECT (pm_gui->checkbutton_install_new),
 		"toggled",
 		G_CALLBACK (cb_pm_checkbutton_install_new_toggled), pm_gui);
-	g_signal_connect (G_OBJECT (pm_gui->dialog_pm),
-		"destroy",
-		G_CALLBACK (pm_dialog_cleanup), pm_gui);
 
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (pm_gui->checkbutton_install_new),
 				      gnm_app_prefs->activate_new_plugins);
@@ -544,6 +542,8 @@ pm_dialog_init (PluginManagerGUI *pm_gui)
 	g_slist_free (sorted_plugin_list);
 
 	cb_pm_selection_changed (pm_gui->selection, pm_gui);
+	g_object_set_data_full (G_OBJECT (pm_gui->dialog_pm),
+		"state", pm_gui, (GDestroyNotify) cb_pm_dialog_free);
 }
 
 static void
@@ -649,7 +649,7 @@ cb_pm_close_clicked (PluginManagerGUI *pm_gui)
 }
 
 void
-dialog_plugin_manager (WorkbookControlGUI *wbcg)
+dialog_plugin_manager (WBCGtk *wbcg)
 {
 	PluginManagerGUI *pm_gui;
 	GladeXML *gui;
@@ -658,9 +658,10 @@ dialog_plugin_manager (WorkbookControlGUI *wbcg)
 	GtkWidget *hbox;
 	GtkTreeViewColumn *column;
 	GtkCellRenderer *rend;
+	GtkWidget *image = NULL;
 
 	g_return_if_fail (wbcg != NULL);
-	g_return_if_fail (IS_WORKBOOK_CONTROL_GUI (wbcg));
+	g_return_if_fail (IS_WBC_GTK (wbcg));
 
 	if (gnumeric_dialog_raise_if_exists (wbcg, PLUGIN_MANAGER_DIALOG_KEY))
 		return;
@@ -679,6 +680,12 @@ dialog_plugin_manager (WorkbookControlGUI *wbcg)
 
 	pm_gui->button_activate_all =
 		GTK_BUTTON (glade_xml_get_widget (gui, "button_activate_all"));
+	/* If we add the following image in glade it does */
+        /* not obey gtk-button-images = 0 */
+	image = g_object_ref (gtk_image_new_from_stock (GTK_STOCK_EXECUTE, 
+							GTK_ICON_SIZE_BUTTON));
+	gtk_button_set_image (pm_gui->button_activate_all, image);
+
 	pm_gui->button_rescan_directories = GTK_BUTTON (glade_xml_get_widget
 						    (gui, "button_rescan_directories"));
 	pm_gui->checkbutton_install_new = GTK_CHECK_BUTTON (glade_xml_get_widget
