@@ -153,6 +153,21 @@ gnm_expr_new_funcall3 (GnmFunc *func,
 	return gnm_expr_new_funcallv (func, 3, argv);
 }
 
+GnmExpr const *
+gnm_expr_new_funcall4 (GnmFunc *func,
+		       GnmExpr const *arg0,
+		       GnmExpr const *arg1,
+		       GnmExpr const *arg2,
+		       GnmExpr const *arg3)
+{
+	GnmExprConstPtr *argv = g_new (GnmExprConstPtr, 4);
+	argv[0] = arg0;
+	argv[1] = arg1;
+	argv[2] = arg2;
+	argv[3] = arg3;
+	return gnm_expr_new_funcallv (func, 4, argv);
+}
+
 
 /***************************************************************************/
 
@@ -645,7 +660,9 @@ value_intersection (GnmValue *v, GnmEvalPos const *pos)
 	gboolean found = FALSE;
 
 	if (v->type == VALUE_ARRAY) {
-		res = value_dup (v->v_array.vals[0][0]);
+		res = (v->v_array.x == 0 || v->v_array.y == 0)
+			? value_new_error_VALUE (NULL)
+			: value_dup (v->v_array.vals[0][0]);
 		value_release (v);
 		return res;
 	}
@@ -1225,6 +1242,9 @@ gnm_expr_eval (GnmExpr const *expr, GnmEvalPos const *pos,
 		value_release (b);
 		return res;
 
+	case GNM_EXPR_OP_PAREN:
+		return gnm_expr_eval (expr->unary.value, pos, flags);
+
 	case GNM_EXPR_OP_PERCENTAGE:
 	case GNM_EXPR_OP_UNARY_NEG:
 	case GNM_EXPR_OP_UNARY_PLUS:
@@ -1476,6 +1496,7 @@ do_expr_as_string (GnmExpr const *expr, int paren_level,
 		guint8 assoc_left, assoc_right;  /* 0: no, 1: yes.  */
 		guint8 is_prefix;                /* for unary operators */
 	} const operations[] = {
+		{ "",   0, 0, 0, 0 }, /* Parentheses for clarity */
 		{ "=",  1, 1, 0, 0 },
 		{ ">",  1, 1, 0, 0 },
 		{ "<",  1, 1, 0, 0 },
@@ -1739,7 +1760,7 @@ reloc_range (GnmExprRelocateInfo const *rinfo,
 			 * deleting rows. Otherwise we #REF! before  we can shorten
 			 * The -1 is safe, origin.start.row == 0 is handled above */
 			if (rinfo->reloc_type == GNM_EXPR_RELOCATE_ROWS &&
-			    rinfo->row_offset >= SHEET_MAX_ROWS)
+			    rinfo->row_offset >= gnm_sheet_get_max_rows (end_sheet))
 				rng->end.row  = rinfo->origin.start.row - 1;
 			else
 				rng->end.row += rinfo->row_offset;
@@ -1759,7 +1780,7 @@ reloc_range (GnmExprRelocateInfo const *rinfo,
 			 * deleting cols. Otherwise we #REF! before  we can shorten.
 			 * The -1 is safe, origin.start.col == 0 is handled above */
 			if (rinfo->reloc_type == GNM_EXPR_RELOCATE_COLS &&
-			    rinfo->col_offset >= SHEET_MAX_COLS)
+			    rinfo->col_offset >= gnm_sheet_get_max_cols (end_sheet))
 				rng->end.col  = rinfo->origin.start.col - 1;
 			else
 				rng->end.col += rinfo->col_offset;
@@ -1801,7 +1822,7 @@ reloc_restore_cellref (RelocInfoInternal const *rinfo,
 		res->sheet = rinfo->details->target_sheet;
 
 	if (!res->col_relative  || rinfo->check_rels) {
-		if (pos->col < 0 || SHEET_MAX_COLS <= pos->col)
+		if (pos->col < 0 || gnm_sheet_get_max_cols (sheet) <= pos->col)
 			return TRUE;
 		res->col = pos->col;
 		if (res->col_relative) {
@@ -1811,7 +1832,7 @@ reloc_restore_cellref (RelocInfoInternal const *rinfo,
 		}
 	}
 	if (!res->row_relative  || rinfo->check_rels) {
-		if (pos->row < 0 || SHEET_MAX_ROWS <= pos->row)
+		if (pos->row < 0 || gnm_sheet_get_max_rows (sheet) <= pos->row)
 			return TRUE;
 		res->row = pos->row;
 		if (res->row_relative) {
@@ -2119,7 +2140,7 @@ cellref_boundingbox (GnmCellRef const *cr, GnmRange *bound)
 {
 	if (cr->col_relative) {
 		if (cr->col >= 0) {
-			int const c = SHEET_MAX_COLS - cr->col - 1;
+			int const c = gnm_sheet_get_max_cols (cr->sheet) - cr->col - 1;
 			if (bound->end.col > c)
 				bound->end.col = c;
 		} else {
@@ -2130,7 +2151,7 @@ cellref_boundingbox (GnmCellRef const *cr, GnmRange *bound)
 	}
 	if (cr->row_relative) {
 		if (cr->row >= 0) {
-			int const r = SHEET_MAX_ROWS - cr->row - 1;
+			int const r = gnm_sheet_get_max_rows (cr->sheet) - cr->row - 1;
 			if (bound->end.row > r)
 				bound->end.row = r;
 		} else {
@@ -2810,7 +2831,7 @@ gnm_expr_top_relocate_sheet (GnmExprTop const *texpr,
 
 	res = gnm_expr_top_relocate (texpr, &rinfo, FALSE);
 	if (!res) {
-		if (gnm_expr_top_get_array_corner (texpr))
+		if (gnm_expr_top_is_array_corner (texpr))
 			res = gnm_expr_top_new (gnm_expr_copy (texpr->expr));
 		else
 			gnm_expr_top_ref ((res = texpr));
@@ -2928,10 +2949,31 @@ gnm_expr_top_get_array_corner (GnmExprTop const *texpr)
 }
 
 gboolean
-gnm_expr_top_is_array_elem (GnmExprTop const *texpr)
+gnm_expr_top_is_array_corner (GnmExprTop const *texpr)
 {
 	g_return_val_if_fail (IS_GNM_EXPR_TOP (texpr), FALSE);
-	return GNM_EXPR_GET_OPER (texpr->expr) == GNM_EXPR_OP_ARRAY_ELEM;
+	return GNM_EXPR_GET_OPER (texpr->expr) == GNM_EXPR_OP_ARRAY_CORNER;
+}
+
+gboolean
+gnm_expr_top_is_array_elem (GnmExprTop const *texpr, int *x, int *y)
+{
+	g_return_val_if_fail (IS_GNM_EXPR_TOP (texpr), FALSE);
+
+	if (GNM_EXPR_GET_OPER (texpr->expr) != GNM_EXPR_OP_ARRAY_ELEM)
+		return FALSE;
+
+	if (x) *x = texpr->expr->array_elem.x;
+	if (y) *y = texpr->expr->array_elem.y;
+	return TRUE;
+}
+
+gboolean
+gnm_expr_top_is_array (GnmExprTop const *texpr)
+{
+	g_return_val_if_fail (IS_GNM_EXPR_TOP (texpr), FALSE);
+	return (GNM_EXPR_GET_OPER (texpr->expr) == GNM_EXPR_OP_ARRAY_ELEM ||
+		GNM_EXPR_GET_OPER (texpr->expr) == GNM_EXPR_OP_ARRAY_CORNER);
 }
 
 GnmExprTop const *
@@ -3020,7 +3062,7 @@ cb_expression_pool_leak (gpointer data, G_GNUC_UNUSED gpointer user)
 	pp.sheet = NULL;
 	pp.wb = NULL;
 	s = gnm_expr_as_string (expr, &pp, gnm_conventions_default);
-	g_printerr ("Leaking expression at %p: %s.\n", expr, s);
+	g_printerr ("Leaking expression at %p: %s.\n", (void *)expr, s);
 	g_free (s);
 }
 #endif
