@@ -1,11 +1,13 @@
 /* vim: set sw=8: -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-#ifndef GNM_PANE_IMPL_H
-#define GNM_PANE_IMPL_H
+#ifndef _GNM_PANE_IMPL_H_
+# define _GNM_PANE_IMPL_H_
 
 #include <src/gnm-pane.h>
 #include <src/gnumeric-simple-canvas.h>
 #include <gtk/gtkimmulticontext.h>
 
+G_BEGIN_DECLS
+
 #define GNM_PANE_MAX_X 1000000
 #define GNM_PANE_MAX_Y 6000000
 
@@ -28,17 +30,17 @@ struct _GnmPane {
 	gboolean   sliding_adjacent_h, sliding_adjacent_v;
 
 	/*  IM */
-	guint      reseting_im :1;	/* quick hack to keep gtk_im_context_reset from starting an edit */
-	guint      preedit_length;
+	guint im_block_edit_start :1; /* see gnm_pane_key_press for details */
+	guint im_first_focus	  :1; /* see gnm_pane_init for details */
+	guint preedit_length;
 	GtkIMContext  *im_context;
 	PangoAttrList *preedit_attrs;
+
 	gboolean insert_decimal;
 
-
-
-
 	int		 index;
 	struct {
+		GtkWidget *alignment;
 		FooCanvas *canvas;
 		ItemBar   *item;
 	} col, row;
@@ -72,4 +74,6 @@ struct _GnmPane {
 	GtkWidget       *size_tip;
 };
 
-#endif /* GNM_PANE_IMPL_H */
+G_END_DECLS
+
+#endif /* _GNM_PANE_IMPL_H_ */
