@@ -48,10 +48,7 @@
 #include <goffice/cut-n-paste/foocanvas/foo-canvas-rect-ellipse.h>
 #include <gsf/gsf-impl-utils.h>
 
-#include <gtk/gtkdnd.h>
-#include <gtk/gtkseparatormenuitem.h>
-#include <gtk/gtkimagemenuitem.h>
-#include <gtk/gtklabel.h>
+#include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 
 #include <string.h>
@@ -325,9 +322,9 @@ gnm_pane_key_mode_sheet (GnmPane *pane, GdkEventKey *event,
 			(*movefn)(scg, r.end.row - sv->edit_pos.row, FALSE, FALSE);
 		} else {
 			/* do the ctrl-home jump to A1 in 2 steps */
-			(*movefn)(scg, -SHEET_MAX_COLS, FALSE, TRUE);
+			(*movefn)(scg, -gnm_sheet_get_max_cols (sheet), FALSE, TRUE);
 			if ((event->state & GDK_CONTROL_MASK) || transition_keys)
-				(*movefn)(scg, -SHEET_MAX_ROWS, FALSE, FALSE);
+				(*movefn)(scg, -gnm_sheet_get_max_rows (sheet), FALSE, FALSE);
 		}
 		break;
 
@@ -412,7 +409,7 @@ gnm_pane_key_mode_sheet (GnmPane *pane, GdkEventKey *event,
 					workbook_cmd_dec_indent (sc->wbc);
 				else
 					workbook_cmd_inc_indent	(sc->wbc);
-			} else {
+			} else if (gnm_app_enter_moves_dir () != GO_DIRECTION_NONE) {
 				gboolean forward = TRUE;
 				gboolean horizontal = TRUE;
 				if (is_enter) {
@@ -427,7 +424,7 @@ gnm_pane_key_mode_sheet (GnmPane *pane, GdkEventKey *event,
 
 				sv_selection_walk_step (sv, forward, horizontal);
 
-				/* invalidate, in case Enter direction gets changes */
+				/* invalidate, in case Enter direction changes */
 				if (is_enter)
 					sv->first_tab_col = -1;
 			}
@@ -525,11 +522,11 @@ gnm_pane_colrow_key_press (SheetControlGUI *scg, GdkEventKey *event,
 			range_init_full_sheet (&target);
 		else {					/* full row */
 			target.start.col = 0;
-			target.end.col = SHEET_MAX_COLS - 1;
+			target.end.col = gnm_sheet_get_max_cols (sv->sheet) - 1;
 		}
 	} else if (event->state & GDK_CONTROL_MASK) {	/* full col */
 		target.start.row = 0;
-		target.end.row = SHEET_MAX_ROWS - 1;
+		target.end.row = gnm_sheet_get_max_rows (sv->sheet) - 1;
 	} else
 		return FALSE;
 
@@ -565,7 +562,8 @@ gnm_pane_key_press (GtkWidget *widget, GdkEventKey *event)
 	}
 
 	/* Object manipulation */
-	if ((scg->selected_objects != NULL || scg->new_object != NULL)) {
+	if (scg->selected_objects != NULL ||
+	     scg->wbcg->new_object != NULL) {
 		if (wbc_gtk_get_guru (scg->wbcg) == NULL  &&
 		    gnm_pane_object_key_press (pane, event))
 			return TRUE;
@@ -587,11 +585,15 @@ gnm_pane_key_press (GtkWidget *widget, GdkEventKey *event)
 		event->keyval == GDK_KP_Decimal ||
 		event->keyval == GDK_KP_Separator;
 
-	if (gtk_im_context_filter_keypress (pane->im_context,event))
+	if (gtk_im_context_filter_keypress (pane->im_context, event))
 		return TRUE;
-	pane->reseting_im = TRUE;
+
+	/* in gtk-2.8 something changed.  gtk_im_context_reset started
+	 * triggering a pre-edit-changed.  We'd end up start and finishing an
+	 * empty edit every time the cursor moved */
+	pane->im_block_edit_start = TRUE;
 	gtk_im_context_reset (pane->im_context);
-	pane->reseting_im = FALSE;
+	pane->im_block_edit_start = FALSE;
 
 	if (gnm_pane_key_mode_sheet (pane, event, allow_rangesel))
 		return TRUE;
@@ -608,7 +610,7 @@ gnm_pane_key_release (GtkWidget *widget, GdkEventKey *event)
 	if (pane->simple.scg->grab_stack > 0)
 		return TRUE;
 
-	if (gtk_im_context_filter_keypress (pane->im_context,event))
+	if (gtk_im_context_filter_keypress (pane->im_context, event))
 		return TRUE;
 	/*
 	 * The status_region normally displays the current edit_pos
@@ -628,7 +630,17 @@ static gint
 gnm_pane_focus_in (GtkWidget *widget, GdkEventFocus *event)
 {
 #ifndef GNM_USE_HILDON
+	/* The first call to focus-in was sometimes the first thing to init the
+	 * imcontext.  In which case the im_context_focus_in would fire a
+	 * preedit-changed, and we would start editing. */
+	GnmPane *pane = GNM_PANE (widget);
+	if (pane->im_first_focus)
+		pane->im_block_edit_start = TRUE;
 	gtk_im_context_focus_in (GNM_PANE (widget)->im_context);
+	if (pane->im_first_focus) {
+		pane->im_first_focus = FALSE;
+		pane->im_block_edit_start = FALSE;
+	}
 #endif
 	return (*GTK_WIDGET_CLASS (parent_klass)->focus_in_event) (widget, event);
 }
@@ -733,10 +745,7 @@ cb_gnm_pane_preedit_changed (GtkIMContext *context, GnmPane *pane)
 		pango_attr_list_unref (pane->preedit_attrs);
 	gtk_im_context_get_preedit_string (pane->im_context, &preedit_string, &pane->preedit_attrs, &cursor_pos);
 
-	/* in gtk-2.8 something changed.  gtk_im_context_reset started
-	 * triggering a pre-edit-changed.  We'd end up start and finishing an
-	 * empty edit every time the cursor moved */
-	if (!pane->reseting_im &&
+	if (!pane->im_block_edit_start &&
 	    !wbcg_is_editing (wbcg) && !wbcg_edit_start (wbcg, TRUE, TRUE)) {
 		gtk_im_context_reset (pane->im_context);
 		pane->preedit_length = 0;
@@ -911,8 +920,9 @@ gnm_pane_init (GnmPane *pane)
 
 	pane->im_context = gtk_im_multicontext_new ();
 	pane->preedit_length = 0;
-	pane->preedit_attrs = NULL;
-	pane->reseting_im = FALSE;
+	pane->preedit_attrs    = NULL;
+	pane->im_block_edit_start = FALSE;
+	pane->im_first_focus = TRUE;
 
 	GTK_WIDGET_SET_FLAGS (canvas, GTK_CAN_FOCUS);
 	GTK_WIDGET_SET_FLAGS (canvas, GTK_CAN_DEFAULT);
@@ -950,7 +960,7 @@ gnm_pane_class_init (GnmPaneClass *klass)
 
 GSF_CLASS (GnmPane, gnm_pane,
 	   gnm_pane_class_init, gnm_pane_init,
-	   GNM_SIMPLE_CANVAS_TYPE);
+	   GNM_SIMPLE_CANVAS_TYPE)
 
 static void
 cb_gnm_pane_header_realized (GtkWidget *widget)
@@ -963,6 +973,7 @@ gnm_pane_header_init (GnmPane *pane, SheetControlGUI *scg,
 		      gboolean is_col_header)
 {
 	Sheet *sheet;
+	GtkWidget *alignment;
 	FooCanvas *canvas = gnm_simple_canvas_new (scg);
 	FooCanvasGroup *group = FOO_CANVAS_GROUP (canvas->root);
 	FooCanvasItem *item = foo_canvas_item_new (group,
@@ -979,10 +990,14 @@ gnm_pane_header_init (GnmPane *pane, SheetControlGUI *scg,
 	if (is_col_header) {
 		pane->col.canvas = canvas;
 		pane->col.item = ITEM_BAR (item);
+		alignment = pane->col.alignment = gtk_alignment_new (0, 1, 1, 0);
 	} else {
 		pane->row.canvas = canvas;
 		pane->row.item = ITEM_BAR (item);
+		alignment = pane->row.alignment = gtk_alignment_new (1, 0, 0, 1);
 	}
+	gtk_container_add (GTK_CONTAINER (alignment), GTK_WIDGET (canvas));
+
 	pane->size_guide.points = NULL;
 	pane->size_guide.start  = NULL;
 	pane->size_guide.guide  = NULL;
@@ -1223,11 +1238,11 @@ gnm_pane_find_col (GnmPane const *pane, int x, int *col_origin)
 			}
 			pixel += tmp;
 		}
-	} while (++col < SHEET_MAX_COLS - 1);
+	} while (++col < gnm_sheet_get_max_cols (sheet) - 1);
 
 	if (col_origin)
 		*col_origin = gnm_pane_x_w2c (pane, pixel);
-	return SHEET_MAX_COLS - 1;
+	return gnm_sheet_get_max_cols (sheet) - 1;
 }
 
 /**
@@ -1273,10 +1288,10 @@ gnm_pane_find_row (GnmPane const *pane, int y, int *row_origin)
 			}
 			pixel += tmp;
 		}
-	} while (++row < SHEET_MAX_ROWS-1);
+	} while (++row < gnm_sheet_get_max_rows (sheet)-1);
 	if (row_origin)
 		*row_origin = pixel;
-	return SHEET_MAX_ROWS-1;
+	return gnm_sheet_get_max_rows (sheet)-1;
 }
 
 /*
@@ -1347,11 +1362,11 @@ gnm_pane_compute_visible_region (GnmPane *pane,
 			pixels = bound;
 		}
 		++col;
-	} while (pixels < width && col < SHEET_MAX_COLS);
+	} while (pixels < width && col < gnm_sheet_get_max_cols (sheet));
 
-	if (col >= SHEET_MAX_COLS) {
-		pane->last_visible.col = SHEET_MAX_COLS-1;
-		pane->last_full.col = SHEET_MAX_COLS-1;
+	if (col >= gnm_sheet_get_max_cols (sheet)) {
+		pane->last_visible.col = gnm_sheet_get_max_cols (sheet)-1;
+		pane->last_full.col = gnm_sheet_get_max_cols (sheet)-1;
 	}
 
 	/* Find out the last visible row and the last fully visible row */
@@ -1379,11 +1394,11 @@ gnm_pane_compute_visible_region (GnmPane *pane,
 			pixels = bound;
 		}
 		++row;
-	} while (pixels < height && row < SHEET_MAX_ROWS);
+	} while (pixels < height && row < gnm_sheet_get_max_rows (sheet));
 
-	if (row >= SHEET_MAX_ROWS) {
-		pane->last_visible.row = SHEET_MAX_ROWS-1;
-		pane->last_full.row = SHEET_MAX_ROWS-1;
+	if (row >= gnm_sheet_get_max_rows (sheet)) {
+		pane->last_visible.row = gnm_sheet_get_max_rows (sheet)-1;
+		pane->last_full.row = gnm_sheet_get_max_rows (sheet)-1;
 	}
 
 	/* Update the scrollbar sizes for the primary pane */
@@ -1428,11 +1443,11 @@ gnm_pane_redraw_range (GnmPane *pane, GnmRange const *r)
 		pane->first_offset.col;
 	y1 = scg_colrow_distance_get (scg, FALSE, pane->first.row, tmp.start.row) +
 		pane->first_offset.row;
-	x2 = (tmp.end.col < (SHEET_MAX_COLS-1))
+	x2 = (tmp.end.col < (gnm_sheet_get_max_cols (sheet)-1))
 		? 4 + 1 + x1 + scg_colrow_distance_get (scg, TRUE,
 							tmp.start.col, tmp.end.col+1)
 		: INT_MAX;
-	y2 = (tmp.end.row < (SHEET_MAX_ROWS-1))
+	y2 = (tmp.end.row < (gnm_sheet_get_max_rows (sheet)-1))
 		? 4 + 1 + y1 + scg_colrow_distance_get (scg, FALSE,
 							tmp.start.row, tmp.end.row+1)
 		: INT_MAX;
@@ -1465,7 +1480,7 @@ gnm_pane_slide_stop (GnmPane *pane)
 }
 
 static int
-col_scroll_step (int dx)
+col_scroll_step (int dx, Sheet *sheet)
 {
 	/* FIXME: get from gdk.  */
 	int dpi_x_this_screen = 90;
@@ -1473,11 +1488,11 @@ col_scroll_step (int dx)
 	double double_dx = dpi_x_this_screen / 3.0;
 	double step = pow (2.0, (dx - start_x) / double_dx);
 
-	return (int) (CLAMP (step, 1.0, SHEET_MAX_COLS / 15.0));
+	return (int) (CLAMP (step, 1.0, gnm_sheet_get_max_cols (sheet) / 15.0));
 }
 
 static int
-row_scroll_step (int dy)
+row_scroll_step (int dy, Sheet *sheet)
 {
 	/* FIXME: get from gdk.  */
 	int dpi_y_this_screen = 90;
@@ -1485,7 +1500,7 @@ row_scroll_step (int dy)
 	double double_dy = dpi_y_this_screen / 8.0;
 	double step = pow (2.0, (dy - start_y) / double_dy);
 
-	return (int) (CLAMP (step, 1.0, SHEET_MAX_ROWS / 15.0));
+	return (int) (CLAMP (step, 1.0, gnm_sheet_get_max_rows (sheet) / 15.0));
 }
 
 static gint
@@ -1499,6 +1514,9 @@ cb_pane_sliding (GnmPane *pane)
 	int col = -1, row = -1;
 	gboolean text_is_rtl = pane->simple.scg->sheet_control.sheet->text_is_rtl;
 	GnmPaneSlideInfo info;
+	Sheet *sheet = 	((SheetControl *) pane->simple.scg)->sheet;
+
+
 
 #if 0
 	g_warning ("slide: %d, %d", pane->sliding_dx, pane->sliding_dy);
@@ -1527,15 +1545,15 @@ cb_pane_sliding (GnmPane *pane)
 
 		if (slide_x) {
 			col = target_pane->last_full.col +
-				col_scroll_step (pane->sliding_dx);
-			if (col >= SHEET_MAX_COLS-1) {
-				col = SHEET_MAX_COLS-1;
+				col_scroll_step (pane->sliding_dx, sheet);
+			if (col >= gnm_sheet_get_max_cols (sheet)-1) {
+				col = gnm_sheet_get_max_cols (sheet)-1;
 				slide_x = FALSE;
 			}
 		}
 	} else if (pane->sliding_dx < 0) {
 		slide_x = TRUE;
-		col = pane0->first.col - col_scroll_step (-pane->sliding_dx);
+		col = pane0->first.col - col_scroll_step (-pane->sliding_dx, sheet);
 
 		if (pane1 != NULL) {
 			if (pane_index == 0 || pane_index == 3) {
@@ -1582,15 +1600,15 @@ cb_pane_sliding (GnmPane *pane)
 
 		if (slide_y) {
 			row = target_pane->last_full.row +
-				row_scroll_step (pane->sliding_dy);
-			if (row >= SHEET_MAX_ROWS-1) {
-				row = SHEET_MAX_ROWS-1;
+				row_scroll_step (pane->sliding_dy, sheet);
+			if (row >= gnm_sheet_get_max_rows (sheet)-1) {
+				row = gnm_sheet_get_max_rows (sheet)-1;
 				slide_y = FALSE;
 			}
 		}
 	} else if (pane->sliding_dy < 0) {
 		slide_y = TRUE;
-		row = pane0->first.row - row_scroll_step (-pane->sliding_dy);
+		row = pane0->first.row - row_scroll_step (-pane->sliding_dy, sheet);
 
 		if (pane3 != NULL) {
 			if (pane_index == 0 || pane_index == 1) {
@@ -2059,9 +2077,9 @@ gnm_pane_size_guide_start (GnmPane *pane, gboolean vert, int colrow, int width)
 			"width-pixels", 1,
 			NULL);
 	else {
-		static char const dat [] = { 0x22, 0x88, 0x22, 0x88, 0x22, 0x88, 0x22, 0x88 };
+		static unsigned char const dat [] = { 0x22, 0x88, 0x22, 0x88, 0x22, 0x88, 0x22, 0x88 };
 		GdkBitmap *stipple = gdk_bitmap_create_from_data (
-			GTK_WIDGET (pane)->window, dat, 8, 8);
+			GTK_WIDGET (pane)->window, (const gchar *)dat, 8, 8);
 		foo_canvas_item_set (pane->size_guide.guide, "fill-stipple", stipple, NULL);
 		g_object_unref (stipple);
 	}
@@ -2250,8 +2268,8 @@ gnm_pane_expr_cursor_bound_set (GnmPane *pane, GnmRange const *r)
 			FOO_CANVAS_GROUP (FOO_CANVAS (pane)->root),
 			item_cursor_get_type (),
 			"SheetControlGUI",	pane->simple.scg,
-			"style",		ITEM_CURSOR_BLOCK,
-			"color",		"red",
+			"style",		ITEM_CURSOR_EXPR_RANGE,
+			"color",		"blue",
 			NULL);
 
 	item_cursor_bound_set (pane->cursor.expr_range, r);
@@ -2491,20 +2509,24 @@ control_point_set_cursor (SheetControlGUI const *scg, FooCanvasItem *ctrl_pt)
 
 	switch (GPOINTER_TO_INT (g_object_get_data (G_OBJECT (ctrl_pt), "index"))) {
 	case 1: invert_v = !invert_v;
+		/* fallthrough */
 	case 6: cursor = invert_v ? GDK_TOP_SIDE : GDK_BOTTOM_SIDE;
 		break;
 
 	case 3: invert_h = !invert_h;
+		/* fallthrough */
 	case 4: cursor = invert_h ? GDK_LEFT_SIDE  : GDK_RIGHT_SIDE;
 		break;
 
 	case 2: invert_h = !invert_h;
+		/* fallthrough */
 	case 0: cursor = invert_v
 			? (invert_h ? GDK_BOTTOM_RIGHT_CORNER : GDK_BOTTOM_LEFT_CORNER)
 			: (invert_h ? GDK_TOP_RIGHT_CORNER : GDK_TOP_LEFT_CORNER);
 		break;
 
 	case 7: invert_h = !invert_h;
+		/* fallthrough */
 	case 5: cursor = invert_v
 			? (invert_h ? GDK_TOP_RIGHT_CORNER : GDK_TOP_LEFT_CORNER)
 			: (invert_h ? GDK_BOTTOM_RIGHT_CORNER : GDK_BOTTOM_LEFT_CORNER);
