@@ -20,9 +20,11 @@
 #include <goffice/utils/go-glib-extras.h>
 #include <goffice/utils/go-libxml-extras.h>
 #include <gsf/gsf-impl-utils.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include <string.h>
 
+#define CXML2C(s) ((char const *)(s))
+
 typedef PluginServiceSimpleClass	PluginServiceFunctionGroupClass;
 struct _PluginServiceFunctionGroup {
 	PluginServiceSimple	base;
@@ -94,7 +96,7 @@ plugin_service_function_group_read_xml (GOPluginService *service, xmlNode *tree,
 		for (node = functions_node->xmlChildrenNode; node != NULL; node = node->next) {
 			gchar *func_name;
 
-			if (strcmp (node->name, "function") != 0 ||
+			if (strcmp (CXML2C (node->name), "function") != 0 ||
 			    (func_name = xml_node_get_cstr (node, "name")) == NULL) {
 				continue;
 			}
@@ -214,8 +216,8 @@ plugin_service_function_group_get_description (GOPluginService *service)
 		: service_function_group->category_name;
 
 	return g_strdup_printf (ngettext (
-			N_("%d function in category \"%s\""),
-			N_("Group of %d functions in category \"%s\""),
+			"%d function in category \"%s\"",
+			"Group of %d functions in category \"%s\"",
 			n_functions),
 		n_functions, category_name);
 }
@@ -313,7 +315,7 @@ static void
 plugin_service_ui_read_xml (GOPluginService *service, xmlNode *tree, ErrorInfo **ret_error)
 {
 	PluginServiceUI *service_ui = GNM_PLUGIN_SERVICE_UI (service);
-	char *file_name;
+	xmlChar *file_name;
 	xmlNode *verbs_node;
 	GSList *actions = NULL;
 
@@ -333,7 +335,7 @@ plugin_service_ui_read_xml (GOPluginService *service, xmlNode *tree, ErrorInfo *
 
 		for (ptr = verbs_node->xmlChildrenNode; ptr != NULL; ptr = ptr->next) {
 			if (xmlIsBlankNode (ptr) || ptr->name == NULL ||
-			    strcmp (ptr->name, "action"))
+			    strcmp (CXML2C (ptr->name), "action"))
 				continue;
 			name  = xml_node_get_cstr (ptr, "name");
 			label = xml_node_get_cstr (ptr, "label");
@@ -404,8 +406,8 @@ plugin_service_ui_get_description (GOPluginService *service)
 	n_actions = g_slist_length (service_ui->actions);
 	return g_strdup_printf (
 		ngettext (
-			N_("User interface with %d action"),
-			N_("User interface with %d actions"),
+			"User interface with %d action",
+			"User interface with %d actions",
 			n_actions),
 		n_actions);
 }
@@ -434,10 +436,6 @@ GSF_CLASS (PluginServiceUI, plugin_service_ui,
 typedef GOPluginLoaderModule	  GnmPluginLoaderModule;
 typedef GOPluginLoaderModuleClass GnmPluginLoaderModuleClass;
 
-#define GNM_PLUGIN_LOADER_MODULE_TYPE (gnm_plugin_loader_module_get_type ())
-#define GNM_PLUGIN_LOADER_MODULE(o)  (G_TYPE_CHECK_INSTANCE_CAST ((o), GNM_PLUGIN_LOADER_MODULE_TYPE, GnmPluginLoaderModule))
-static GType gnm_plugin_loader_module_get_type (void);
-
 /*
  * Service - function_group
  */
@@ -558,7 +556,8 @@ gnm_plugin_loader_module_func_exec_action (GOPluginService *service,
 		return;
 	}
 	action_index = GPOINTER_TO_INT (action_index_ptr);
-	loader_data->module_ui_actions_array [action_index].handler (action, wbc);
+	if (NULL != loader_data->module_ui_actions_array [action_index].handler)
+		(*loader_data->module_ui_actions_array [action_index].handler) (action, wbc);
 }
 
 static void
@@ -637,7 +636,7 @@ go_plugin_loader_module_iface_init (GOPluginLoaderClass *iface)
 	iface->service_unload = gplm_service_unload;
 }
 
-static GSF_CLASS_FULL (GnmPluginLoaderModule, gnm_plugin_loader_module,
+GSF_CLASS_FULL (GnmPluginLoaderModule, gnm_plugin_loader_module,
            NULL, NULL, NULL, NULL,
            NULL, GO_PLUGIN_LOADER_MODULE_TYPE, 0,
 	   GSF_INTERFACE (go_plugin_loader_module_iface_init, GO_PLUGIN_LOADER_TYPE))
