@@ -45,8 +45,9 @@
 #include <fcntl.h>
 #include <time.h>
 #include <string.h>
+#include <locale.h>
 
-#ifdef WITH_GNOME
+#ifdef GNM_WITH_GNOME
 #include <bonobo/bonobo-main.h>
 #include <bonobo/bonobo-ui-main.h>
 #include <libgnome/gnome-program.h>
@@ -55,7 +56,7 @@
 #include <libgnomeui/gnome-authentication-manager.h>
 #endif
 
-#ifdef USE_HILDON
+#ifdef GNM_USE_HILDON
 #include <libosso.h>
 #endif
 
@@ -188,7 +189,7 @@ gnumeric_arg_parse (int argc, char **argv)
 	g_option_context_add_main_entries (ocontext, gnumeric_options, GETTEXT_PACKAGE);
 	g_option_context_add_group	  (ocontext, gnm_get_option_group ());
 
-#ifdef WITH_GNOME
+#ifdef GNM_WITH_GNOME
 #ifndef GNOME_PARAM_GOPTION_CONTEXT
 	/*
 	 * Bummer.  We cannot make gnome_program_init handle our args so
@@ -238,7 +239,7 @@ gnumeric_arg_parse (int argc, char **argv)
 
 	if (!funcdump) {
 		gtk_init (&argc, &argv);
-#ifdef WITH_GNOME
+#ifdef GNM_WITH_GNOME
 		gnome_authentication_manager_init ();
 #endif
 	}
@@ -258,7 +259,7 @@ store_plugin_state (void)
 	g_slist_free (active_plugins);
 }
 
-static int
+static gboolean
 cb_kill_wbcg (WBCGtk *wbcg)
 {
 	gboolean still_open = wbc_gtk_close (wbcg);
@@ -266,6 +267,26 @@ cb_kill_wbcg (WBCGtk *wbcg)
 	return FALSE;
 }
 
+static gboolean
+pathetic_qt_workaround (void)
+{
+	/*
+	 * When using with the Qt theme, the qt library will be initialized
+	 * somewhere around the time the first widget is created or maybe
+	 * realized.  That code literally does
+	 *
+	 *        setlocale( LC_NUMERIC, "C" );	// make sprintf()/scanf() work
+	 *
+	 * I am not kidding.  It seems like we can fix this by re-setting the
+	 * proper locale when the gui comes up.
+	 *
+	 * See bug 512752, for example.
+	 */
+	setlocale (LC_ALL, "");
+	return FALSE;
+}
+
+
 static void
 check_pango_attr_list_splice_bug (void)
 {
@@ -307,7 +328,12 @@ main (int argc, char const **argv)
 	IOContext *ioc;
 	WorkbookView *wbv;
 	GSList *wbcgs_to_kill = NULL;
-#ifdef USE_HILDON
+
+#ifdef G_OS_WIN32
+	gboolean has_console;
+#endif
+
+#ifdef GNM_USE_HILDON
 	osso_context_t * osso_context;
 #endif
 
@@ -315,7 +341,7 @@ main (int argc, char const **argv)
 	argv = gnm_pre_parse_init (argc, argv);
 
 #ifdef G_OS_WIN32
-	gboolean has_console = FALSE;
+	has_console = FALSE;
 	{
 		typedef BOOL (CALLBACK* LPFNATTACHCONSOLE)(DWORD);
 		LPFNATTACHCONSOLE MyAttachConsole;
@@ -334,8 +360,8 @@ main (int argc, char const **argv)
 	}
 #endif
 
-#ifdef USE_HILDON
-	osso_context = osso_initialize ("gnumeric", GNUMERIC_VERSION, TRUE, NULL);
+#ifdef GNM_USE_HILDON
+	osso_context = osso_initialize ("gnumeric", GNM_VERSION_FULL, TRUE, NULL);
 #endif
 
 	gnumeric_arg_parse (argc, (char **)argv);
@@ -353,6 +379,7 @@ main (int argc, char const **argv)
 	if (with_gui) {
 		ioc = IO_CONTEXT (g_object_new (TYPE_IO_CONTEXT_GTK, NULL));
 		handle_paint_events ();
+		pathetic_qt_workaround ();
 	} else {
 		/* TODO: Make this inconsistency go away */
 		GOCmdContext *cc = cmd_context_stderr_new ();
@@ -371,7 +398,7 @@ main (int argc, char const **argv)
 	g_set_application_name (_("Gnumeric Spreadsheet"));
 	gnm_plugins_init (GO_CMD_CONTEXT (ioc));
 
-#ifdef WITH_GNOME
+#ifdef GNM_WITH_GNOME
 	bonobo_activate ();
 #endif
 	if (startup_files) {
@@ -422,7 +449,7 @@ main (int argc, char const **argv)
 				break; /* Don't load any more workbooks */
 		}
 	}
-	/* FIXME: May be we should quit here if we were asked to open
+	/* FIXME: Maybe we should quit here if we were asked to open
 	   files and failed to do so. */
 
 	/* If we were intentionally short circuited exit now */
@@ -445,7 +472,9 @@ main (int argc, char const **argv)
 			warn_about_ancient_gnumerics (g_get_prgname(), ioc);
 		}
 		g_object_unref (ioc);
-#ifdef WITH_GNOME
+
+		g_idle_add ((GSourceFunc)pathetic_qt_workaround, NULL);
+#ifdef GNM_WITH_GNOME
 		bonobo_main ();
 #else
 		gtk_main ();
@@ -455,7 +484,7 @@ main (int argc, char const **argv)
 		g_slist_foreach (wbcgs_to_kill, (GFunc)cb_kill_wbcg, NULL);
 	}
 
-#ifdef USE_HILDON
+#ifdef GNM_USE_HILDON
 	osso_deinitialize (osso_context);
 #endif
 
@@ -464,7 +493,7 @@ main (int argc, char const **argv)
 	store_plugin_state ();
 	gnm_shutdown ();
 
-#ifdef WITH_GNOME
+#ifdef GNM_WITH_GNOME
 	bonobo_ui_debug_shutdown ();
 #elif defined(G_OS_WIN32)
 	if (has_console) {
