@@ -490,11 +490,11 @@ gnm_style_dup (GnmStyle const *src)
 
 /**
  * gnm_style_new_merged :
- * @src     : #GnmStyle
+ * @base    : #GnmStyle
  * @overlay : #GnmStyle
  *
  * A new GnmStyle that contains any elements of @overlay that are set, and uses
- * @src for anything that is not set in @overlay.
+ * @base for anything that is not set in @overlay.
  *
  * Returns A ref to a new GnmStyle.
  **/
@@ -506,7 +506,12 @@ gnm_style_new_merged (GnmStyle const *base, GnmStyle const *overlay)
 
 	new_style->ref_count = 1;
 	for (i = 0; i < MSTYLE_ELEMENT_MAX; i++) {
-		elem_assign_contents (new_style, elem_is_set (overlay, i) ? overlay : base, i);
+		if (elem_is_set (overlay, i))
+			elem_assign_contents (new_style, overlay, i);
+		else if (elem_is_set (base, i))
+			elem_assign_contents (new_style, base, i);
+		else
+			continue;
 		elem_set (new_style, i);
 		elem_changed (new_style, i);
 	}
@@ -593,13 +598,17 @@ link_pattern_color (GnmStyle *style, GnmColor *auto_color, gboolean make_copy)
 static GnmStyle *
 link_border_colors (GnmStyle *style, GnmColor *auto_color, gboolean make_copy)
 {
-	GnmBorder *border;
-	GnmColor *color;
 	int i;
 
 	for (i = MSTYLE_BORDER_TOP ; i <= MSTYLE_BORDER_DIAGONAL ; ++i) {
 		if (elem_is_set (style, i)) {
-			border = style->borders[i- MSTYLE_BORDER_TOP];
+			GnmBorder *border =
+				style->borders[i- MSTYLE_BORDER_TOP];
+			GnmColor *color;
+
+			if (!border)
+				continue;
+
 			color = border->color;
 			if (color->is_auto && auto_color != color) {
 				GnmBorder *new_border;
@@ -811,14 +820,17 @@ gnm_style_unset_element (GnmStyle *style, GnmStyleElement elem)
  * @base    : #GnmStyle
  * @overlay : #GnmStyle
  *
- * Applies any elements of @overlay that are set to @base.
+ * Applies all active elements of @overlay onto @base.
  **/
 void
 gnm_style_merge (GnmStyle *base, GnmStyle const *overlay)
 {
 	unsigned i;
+	if (base == overlay)
+		return;
 	for (i = 0; i < MSTYLE_ELEMENT_MAX; i++)
 		if (elem_is_set (overlay, i)) {
+			elem_clear_contents (base, i);
 			elem_assign_contents (base, overlay, i);
 			elem_changed (base, i);
 		}
@@ -1770,13 +1782,13 @@ gnm_style_dump (GnmStyle const *style)
 	if (elem_is_set (style, MSTYLE_CONTENTS_HIDDEN))
 		g_printerr ("\thidden %d\n", style->contents_hidden);
 	if (elem_is_set (style, MSTYLE_VALIDATION))
-		g_printerr ("\tvalidation %p\n", style->validation);
+		g_printerr ("\tvalidation %p\n", (void *)style->validation);
 	if (elem_is_set (style, MSTYLE_HLINK))
-		g_printerr ("\thlink %p\n", style->hlink);
+		g_printerr ("\thlink %p\n", (void *)style->hlink);
 	if (elem_is_set (style, MSTYLE_INPUT_MSG))
-		g_printerr ("\tinput msg %p\n", style->input_msg);
+		g_printerr ("\tinput msg %p\n", (void *)style->input_msg);
 	if (elem_is_set (style, MSTYLE_CONDITIONS))
-		g_printerr ("\tconditions %p\n", style->conditions);
+		g_printerr ("\tconditions %p\n", (void *)style->conditions);
 }
 
 /* ------------------------------------------------------------------------- */
@@ -1797,7 +1809,7 @@ static void
 cb_gnm_style_pool_leak (gpointer data, gpointer user)
 {
 	GnmStyle *style = data;
-	g_printerr ("Leaking style at %p.\n", style);
+	g_printerr ("Leaking style at %p.\n", (void *)style);
 	gnm_style_dump (style);
 }
 #endif
