@@ -321,13 +321,7 @@ typedef struct {
 	/* if arg_sep conflicts with array_col_sep or array_row_sep */
 	int in_array_sep_is;	/* token id */
 
-	/* flags */
-	gboolean force_absolute_col_references;
-	gboolean force_absolute_row_references;
-	gboolean force_explicit_sheet_references;
-	gboolean unknown_names_are_strings;
-	gboolean unknown_names_are_invalid;
-
+	GnmExprParseFlags     flags;
 	GnmConventions const *convs;
 
 	/* dynamic state */
@@ -358,7 +352,7 @@ report_err (ParserState *state, GError *err,
 static GnmExpr *
 fold_negative_constant (GnmExpr *expr)
 {
-	if (GNM_EXPR_GET_OPER (expr) == GNM_EXPR_OP_CONSTANT) {
+	if (expr && GNM_EXPR_GET_OPER (expr) == GNM_EXPR_OP_CONSTANT) {
 		GnmValue *v = (GnmValue *)expr->constant.value;
 		
 		if (VALUE_IS_FLOAT (v)) {
@@ -376,7 +370,7 @@ fold_negative_constant (GnmExpr *expr)
 static GnmExpr *
 fold_positive_constant (GnmExpr *expr)
 {
-	if (GNM_EXPR_GET_OPER (expr) == GNM_EXPR_OP_CONSTANT) {
+	if (expr && GNM_EXPR_GET_OPER (expr) == GNM_EXPR_OP_CONSTANT) {
 		const GnmValue *v = expr->constant.value;
 		if (VALUE_IS_FLOAT (v))
 			return expr;
@@ -388,6 +382,8 @@ fold_positive_constant (GnmExpr *expr)
 static GnmExpr *
 build_unary_op (GnmExprOp op, GnmExpr *expr)
 {
+	if (!expr) return NULL;
+
 	unregister_allocation (expr);
 	return register_expr_allocation (gnm_expr_new_unary (op, expr));
 }
@@ -395,6 +391,8 @@ build_unary_op (GnmExprOp op, GnmExpr *expr)
 static GnmExpr *
 build_binop (GnmExpr *l, GnmExprOp op, GnmExpr *r)
 {
+	if (!l || !r) return NULL;
+
 	unregister_allocation (r);
 	unregister_allocation (l);
 	return register_expr_allocation (gnm_expr_new_binary (l, op, r));
@@ -405,6 +403,8 @@ build_logical (GnmExpr *l, gboolean is_and, GnmExpr *r)
 {
 	static GnmFunc *and_func = NULL, *or_func = NULL;
 
+	if (!l || !r) return NULL;
+
 	if (and_func == NULL)
 		and_func = gnm_func_lookup ("AND", NULL);
 	if (or_func == NULL)
@@ -420,6 +420,9 @@ static GnmExpr *
 build_not (GnmExpr *expr)
 {
 	static GnmFunc *not_func = NULL;
+
+	if (!expr) return NULL;
+
 	if (not_func == NULL)
 		not_func = gnm_func_lookup ("NOT", NULL);
 	unregister_allocation (expr);
@@ -427,12 +430,16 @@ build_not (GnmExpr *expr)
 		(gnm_expr_new_funcall1 (not_func, expr));
 }
 
+/*
+ * Build an array expression.
+ *
+ * Returns NULL on failure.  Caller must YYERROR in that case.
+ */
 static GnmExpr *
 build_array (GSList *cols)
 {
 	GnmValue *array;
-	GSList *row;
-	int x, mx, y;
+	int mx, y;
 
 	if (!cols) {
 		report_err (state, g_error_new (1, PERR_INVALID_EMPTY,
@@ -441,24 +448,18 @@ build_array (GSList *cols)
 		return NULL;
 	}
 
-	mx  = 0;
-	row = cols->data;
-	while (row) {
-		mx++;
-		row = row->next;
-	}
-
+	mx = g_list_length (cols->data);
 	array = value_new_array_empty (mx, g_slist_length (cols));
 
 	y = 0;
 	while (cols) {
-		row = cols->data;
-		x = 0;
+		GSList *row = cols->data;
+		int x = 0;
 		while (row && x < mx) {
-			GnmExpr    *expr = row->data;
+			GnmExpr const *expr = row->data;
 			GnmValue const *v = expr->constant.value;
 
-			g_assert (GNM_EXPR_GET_OPER (expr) == GNM_EXPR_OP_CONSTANT);
+			g_assert (expr && GNM_EXPR_GET_OPER (expr) == GNM_EXPR_OP_CONSTANT);
 
 			value_array_set (array, x, y, value_dup (v));
 
@@ -480,6 +481,11 @@ build_array (GSList *cols)
 	return register_expr_allocation (gnm_expr_new_constant (array));
 }
 
+/*
+ * Build a range constructor.
+ *
+ * Returns NULL on failure.  Caller must YYERROR in that case.
+ */
 static GnmExpr *
 build_range_ctor (GnmExpr *l, GnmExpr *r, GnmExpr *validate)
 {
@@ -495,9 +501,16 @@ build_range_ctor (GnmExpr *l, GnmExpr *r, GnmExpr *validate)
 	return build_binop (l, GNM_EXPR_OP_RANGE_CTOR, r);
 }
 
+/*
+ * Build an intersection expression.
+ *
+ * Returns NULL on failure.  Caller must YYERROR in that case.
+ */
 static GnmExpr *
 build_intersect (GnmExpr *l, GnmExpr *r)
 {
+	if (!l || !r) return NULL;
+
 	if (gnm_expr_is_rangeref (l) && gnm_expr_is_rangeref (r))
 		return build_binop (l, GNM_EXPR_OP_INTERSECT, r);
 	report_err (state, g_error_new (1, PERR_SET_CONTENT_MUST_BE_RANGE,
@@ -506,6 +519,11 @@ build_intersect (GnmExpr *l, GnmExpr *r)
 	return NULL;
 }
 
+/*
+ * Build a set expression.
+ *
+ * Returns NULL on failure.  Caller must YYERROR in that case.
+ */
 static GnmExpr *
 build_set (GnmExprList *list)
 {
@@ -513,7 +531,7 @@ build_set (GnmExprList *list)
 	GnmExprList *ptr;
 	for (ptr = list; ptr != NULL ; ptr = ptr->next) {
 		GnmExpr const *expr = ptr->data;
-		if (!gnm_expr_is_rangeref (expr)) {
+		if (!expr || !gnm_expr_is_rangeref (expr)) {
 			report_err (state, g_error_new (1, PERR_SET_CONTENT_MUST_BE_RANGE,
 				_("All entries in the set must be references")),
 				state->ptr, 0);
@@ -563,13 +581,13 @@ parser_simple_val_or_name (GnmExpr *str_expr)
 	if (v == NULL) {
 		GnmNamedExpr *nexpr = expr_name_lookup (state->pos, str);
 		if (nexpr == NULL) {
-			if (state->unknown_names_are_invalid) {
+			if (state->flags & GNM_EXPR_PARSE_UNKNOWN_NAMES_ARE_INVALID) {
 				report_err (state, g_error_new (1, PERR_UNKNOWN_NAME,
 								_("Name '%s' does not exist"),
 								str),
 					    state->ptr, 0);
 				res = NULL;
-			} else if (state->unknown_names_are_strings) {
+			} else if (state->flags & GNM_EXPR_PARSE_UNKNOWN_NAMES_ARE_STRINGS) {
 				res = gnm_expr_new_constant (value_new_string (str));
 			} else {
 				GnmParsePos pp = *state->pos;
@@ -641,7 +659,7 @@ int yyparse (void);
 
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 typedef union YYSTYPE
-#line 507 "parser.y"
+#line 525 "parser.y"
 {
 	GnmExpr		*expr;
 	GnmValue	*value;
@@ -650,8 +668,8 @@ typedef union YYSTYPE
 	Sheet		*sheet;
 	Workbook	*wb;
 }
-/* Line 193 of yacc.c.  */
-#line 655 "parser.c"
+/* Line 187 of yacc.c.  */
+#line 673 "parser.c"
 	YYSTYPE;
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
@@ -664,7 +682,7 @@ typedef union YYSTYPE
 
 
 /* Line 216 of yacc.c.  */
-#line 668 "parser.c"
+#line 686 "parser.c"
 
 #ifdef short
 # undef short
@@ -969,12 +987,12 @@ static const yytype_int8 yyrhs[] =
 /* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
 static const yytype_uint16 yyrline[] =
 {
-       0,   535,   535,   541,   549,   555,   558,   559,   560,   564,
-     565,   566,   567,   568,   569,   570,   571,   572,   573,   574,
-     575,   576,   577,   578,   579,   584,   588,   592,   593,   595,
-     615,   621,   622,   640,   661,   678,   679,   682,   714,   723,
-     735,   736,   737,   741,   745,   750,   755,   766,   776,   779,
-     780,   785,   790,   794,   795,   800,   808,   813
+       0,   553,   553,   559,   567,   573,   576,   577,   578,   582,
+     583,   584,   585,   586,   587,   588,   589,   590,   591,   592,
+     593,   594,   595,   596,   597,   602,   606,   610,   611,   613,
+     631,   638,   639,   657,   678,   695,   696,   699,   731,   740,
+     752,   753,   757,   761,   765,   770,   775,   786,   796,   799,
+     800,   805,   810,   814,   815,   820,   828,   833
 };
 #endif
 
@@ -1957,7 +1975,7 @@ yyreduce:
   switch (yyn)
     {
         case 2:
-#line 535 "parser.y"
+#line 553 "parser.y"
     {
 		unregister_allocation ((yyvsp[(2) - (2)].expr));
 		unregister_allocation ((yyvsp[(1) - (2)].list));
@@ -1966,7 +1984,7 @@ yyreduce:
     break;
 
   case 3:
-#line 541 "parser.y"
+#line 559 "parser.y"
     {
 		if (state->result != NULL) {
 			gnm_expr_list_unref (state->result);
@@ -1976,7 +1994,7 @@ yyreduce:
     break;
 
   case 4:
-#line 549 "parser.y"
+#line 567 "parser.y"
     {
 	       unregister_allocation ((yyvsp[(2) - (3)].expr));
 	       unregister_allocation ((yyvsp[(1) - (3)].list));
@@ -1986,22 +2004,22 @@ yyreduce:
     break;
 
   case 5:
-#line 555 "parser.y"
+#line 573 "parser.y"
     { (yyval.list) = NULL; register_expr_list_allocation ((yyval.list)); }
     break;
 
   case 6:
-#line 558 "parser.y"
+#line 576 "parser.y"
     { (yyval.expr) = (yyvsp[(1) - (1)].expr); }
     break;
 
   case 7:
-#line 559 "parser.y"
+#line 577 "parser.y"
     { (yyval.expr) = (yyvsp[(1) - (1)].expr); }
     break;
 
   case 8:
-#line 560 "parser.y"
+#line 578 "parser.y"
     {
 		(yyval.expr) = parser_simple_val_or_name ((yyvsp[(1) - (1)].expr));
 		if ((yyval.expr) == NULL) { YYERROR; }
@@ -2009,82 +2027,82 @@ yyreduce:
     break;
 
   case 9:
-#line 564 "parser.y"
+#line 582 "parser.y"
     { (yyval.expr) = (yyvsp[(1) - (1)].expr); }
     break;
 
   case 10:
-#line 565 "parser.y"
+#line 583 "parser.y"
     { (yyval.expr) = build_binop ((yyvsp[(1) - (3)].expr), GNM_EXPR_OP_ADD,	(yyvsp[(3) - (3)].expr)); }
     break;
 
   case 11:
-#line 566 "parser.y"
+#line 584 "parser.y"
     { (yyval.expr) = build_binop ((yyvsp[(1) - (3)].expr), GNM_EXPR_OP_SUB,	(yyvsp[(3) - (3)].expr)); }
     break;
 
   case 12:
-#line 567 "parser.y"
+#line 585 "parser.y"
     { (yyval.expr) = build_binop ((yyvsp[(1) - (3)].expr), GNM_EXPR_OP_MULT,	(yyvsp[(3) - (3)].expr)); }
     break;
 
   case 13:
-#line 568 "parser.y"
+#line 586 "parser.y"
     { (yyval.expr) = build_binop ((yyvsp[(1) - (3)].expr), GNM_EXPR_OP_DIV,	(yyvsp[(3) - (3)].expr)); }
     break;
 
   case 14:
-#line 569 "parser.y"
+#line 587 "parser.y"
     { (yyval.expr) = build_binop ((yyvsp[(1) - (3)].expr), GNM_EXPR_OP_EXP,	(yyvsp[(3) - (3)].expr)); }
     break;
 
   case 15:
-#line 570 "parser.y"
+#line 588 "parser.y"
     { (yyval.expr) = build_binop ((yyvsp[(1) - (3)].expr), GNM_EXPR_OP_CAT,	(yyvsp[(3) - (3)].expr)); }
     break;
 
   case 16:
-#line 571 "parser.y"
+#line 589 "parser.y"
     { (yyval.expr) = build_binop ((yyvsp[(1) - (3)].expr), GNM_EXPR_OP_EQUAL,	(yyvsp[(3) - (3)].expr)); }
     break;
 
   case 17:
-#line 572 "parser.y"
+#line 590 "parser.y"
     { (yyval.expr) = build_binop ((yyvsp[(1) - (3)].expr), GNM_EXPR_OP_LT,		(yyvsp[(3) - (3)].expr)); }
     break;
 
   case 18:
-#line 573 "parser.y"
+#line 591 "parser.y"
     { (yyval.expr) = build_binop ((yyvsp[(1) - (3)].expr), GNM_EXPR_OP_GT,		(yyvsp[(3) - (3)].expr)); }
     break;
 
   case 19:
-#line 574 "parser.y"
+#line 592 "parser.y"
     { (yyval.expr) = build_binop ((yyvsp[(1) - (3)].expr), GNM_EXPR_OP_GTE,	(yyvsp[(3) - (3)].expr)); }
     break;
 
   case 20:
-#line 575 "parser.y"
+#line 593 "parser.y"
     { (yyval.expr) = build_binop ((yyvsp[(1) - (3)].expr), GNM_EXPR_OP_NOT_EQUAL,	(yyvsp[(3) - (3)].expr)); }
     break;
 
   case 21:
-#line 576 "parser.y"
+#line 594 "parser.y"
     { (yyval.expr) = build_binop ((yyvsp[(1) - (3)].expr), GNM_EXPR_OP_LTE,	(yyvsp[(3) - (3)].expr)); }
     break;
 
   case 22:
-#line 577 "parser.y"
+#line 595 "parser.y"
     { (yyval.expr) = build_logical ((yyvsp[(1) - (3)].expr), TRUE,	(yyvsp[(3) - (3)].expr)); }
     break;
 
   case 23:
-#line 578 "parser.y"
+#line 596 "parser.y"
     { (yyval.expr) = build_logical ((yyvsp[(1) - (3)].expr), FALSE, (yyvsp[(3) - (3)].expr)); }
     break;
 
   case 24:
-#line 579 "parser.y"
+#line 597 "parser.y"
     {
 		(yyval.expr) = build_intersect ((yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr));
 		if ((yyval.expr) == NULL) { YYERROR; }
@@ -2092,7 +2110,7 @@ yyreduce:
     break;
 
   case 25:
-#line 584 "parser.y"
+#line 602 "parser.y"
     {
 		GnmExpr *tmp = fold_negative_constant ((yyvsp[(2) - (2)].expr));
 		(yyval.expr) = tmp ? tmp : build_unary_op (GNM_EXPR_OP_UNARY_NEG, (yyvsp[(2) - (2)].expr));
@@ -2100,7 +2118,7 @@ yyreduce:
     break;
 
   case 26:
-#line 588 "parser.y"
+#line 606 "parser.y"
     {
 		/* Don't fold here.  */
 		(yyval.expr) = build_unary_op (GNM_EXPR_OP_UNARY_PLUS, (yyvsp[(2) - (2)].expr));
@@ -2108,17 +2126,17 @@ yyreduce:
     break;
 
   case 27:
-#line 592 "parser.y"
+#line 610 "parser.y"
     { (yyval.expr) = build_not ((yyvsp[(2) - (2)].expr)); }
     break;
 
   case 28:
-#line 593 "parser.y"
+#line 611 "parser.y"
     { (yyval.expr) = build_unary_op (GNM_EXPR_OP_PERCENTAGE, (yyvsp[(1) - (2)].expr)); }
     break;
 
   case 29:
-#line 595 "parser.y"
+#line 613 "parser.y"
     {
 		if ((yyvsp[(2) - (3)].list) == NULL) {
 			report_err (state, g_error_new (1, PERR_INVALID_EMPTY,
@@ -2128,30 +2146,29 @@ yyreduce:
 		} else {
 			if ((yyvsp[(2) - (3)].list)->next == NULL) {
 				unregister_allocation ((yyvsp[(2) - (3)].list));
-				(yyval.expr) = register_expr_allocation ((yyvsp[(2) - (3)].list)->data);
+				(yyval.expr) = register_expr_allocation (gnm_expr_new_unary (GNM_EXPR_OP_PAREN, (yyvsp[(2) - (3)].list)->data));
 				/* NOTE : free list not content */
 				gnm_expr_list_free ((yyvsp[(2) - (3)].list));
 			} else {
 				(yyval.expr) = build_set ((yyvsp[(2) - (3)].list));
-				if ((yyval.expr) == NULL) {
-					YYERROR;
-				}
+				if ((yyval.expr) == NULL) { YYERROR; }
 			}
 		}
 	}
     break;
 
   case 30:
-#line 615 "parser.y"
+#line 631 "parser.y"
     {
 		unregister_allocation ((yyvsp[(2) - (3)].list));
 		(yyval.expr) = build_array ((yyvsp[(2) - (3)].list));
 		free_expr_list_list ((yyvsp[(2) - (3)].list));
+		if ((yyval.expr) == NULL) { YYERROR; }
 	}
     break;
 
   case 32:
-#line 622 "parser.y"
+#line 639 "parser.y"
     {
 		GnmNamedExpr *nexpr = NULL;
 		char const *name = (yyvsp[(2) - (2)].expr)->constant.value->v_str.val->str;
@@ -2173,7 +2190,7 @@ yyreduce:
     break;
 
   case 33:
-#line 640 "parser.y"
+#line 657 "parser.y"
     {
 		GnmNamedExpr *nexpr = NULL;
 		char const *name = (yyvsp[(2) - (2)].expr)->constant.value->v_str.val->str;
@@ -2187,8 +2204,8 @@ yyreduce:
 			(yyval.expr) = register_expr_allocation (gnm_expr_new_name (nexpr, NULL, (yyvsp[(1) - (2)].wb)));
 		} else {
 			report_err (state, g_error_new (1, PERR_UNKNOWN_NAME,
-				_("Name '%s' does not exist in sheet '%s'"),
-						name, pos.sheet->name_quoted),
+				_("Name '%s' does not exist in workbook"),
+							name),
 				state->ptr, strlen (name));
 			YYERROR;
 		}
@@ -2196,7 +2213,7 @@ yyreduce:
     break;
 
   case 34:
-#line 661 "parser.y"
+#line 678 "parser.y"
     {
 		char const *name = (yyvsp[(1) - (4)].expr)->constant.value->v_str.val->str;
 		GnmExpr const *f_call = (*state->convs->input.func) (
@@ -2215,7 +2232,7 @@ yyreduce:
     break;
 
   case 37:
-#line 682 "parser.y"
+#line 699 "parser.y"
     {
 		char const *wb_name = (yyvsp[(2) - (3)].expr)->constant.value->v_str.val->str;
 		Workbook *ref_wb = state->pos
@@ -2246,7 +2263,7 @@ yyreduce:
     break;
 
   case 38:
-#line 714 "parser.y"
+#line 731 "parser.y"
     {
 		Sheet *sheet = parser_sheet_by_name (state->pos->wb, (yyvsp[(1) - (2)].expr));
 		if (sheet != NULL) {
@@ -2259,7 +2276,7 @@ yyreduce:
     break;
 
   case 39:
-#line 723 "parser.y"
+#line 740 "parser.y"
     {
 		Workbook *wb = (yyvsp[(1) - (3)].wb);
 		Sheet *sheet = parser_sheet_by_name (wb, (yyvsp[(2) - (3)].expr));
@@ -2273,17 +2290,20 @@ yyreduce:
     break;
 
   case 40:
-#line 735 "parser.y"
+#line 752 "parser.y"
     { (yyval.expr) = (yyvsp[(1) - (1)].expr); }
     break;
 
   case 41:
-#line 736 "parser.y"
-    { (yyval.expr) = build_range_ctor ((yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr), NULL); }
+#line 753 "parser.y"
+    {
+		(yyval.expr) = build_range_ctor ((yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr), NULL);
+		if ((yyval.expr) == NULL) { YYERROR; }
+	}
     break;
 
   case 42:
-#line 737 "parser.y"
+#line 757 "parser.y"
     {
 		(yyval.expr) = build_range_ctor ((yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr), (yyvsp[(1) - (3)].expr));
 		if ((yyval.expr) == NULL) { YYERROR; }
@@ -2291,7 +2311,7 @@ yyreduce:
     break;
 
   case 43:
-#line 741 "parser.y"
+#line 761 "parser.y"
     {
 		(yyval.expr) = build_range_ctor ((yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr), (yyvsp[(3) - (3)].expr));
 		if ((yyval.expr) == NULL) { YYERROR; }
@@ -2299,14 +2319,14 @@ yyreduce:
     break;
 
   case 44:
-#line 745 "parser.y"
+#line 765 "parser.y"
     {
 		(yyval.expr) = build_binop ((yyvsp[(1) - (3)].expr), GNM_EXPR_OP_RANGE_CTOR, (yyvsp[(3) - (3)].expr));
 	}
     break;
 
   case 45:
-#line 750 "parser.y"
+#line 770 "parser.y"
     {
 		unregister_allocation ((yyvsp[(1) - (1)].expr));
 		(yyval.list) = gnm_expr_list_prepend (NULL, (yyvsp[(1) - (1)].expr));
@@ -2315,7 +2335,7 @@ yyreduce:
     break;
 
   case 46:
-#line 755 "parser.y"
+#line 775 "parser.y"
     {
 		GSList *tmp = (yyvsp[(3) - (3)].list);
 		unregister_allocation ((yyvsp[(3) - (3)].list));
@@ -2330,7 +2350,7 @@ yyreduce:
     break;
 
   case 47:
-#line 766 "parser.y"
+#line 786 "parser.y"
     {
 		GSList *tmp = (yyvsp[(2) - (2)].list);
 		unregister_allocation ((yyvsp[(2) - (2)].list));
@@ -2344,17 +2364,17 @@ yyreduce:
     break;
 
   case 48:
-#line 776 "parser.y"
+#line 796 "parser.y"
     { (yyval.list) = NULL; }
     break;
 
   case 49:
-#line 779 "parser.y"
+#line 799 "parser.y"
     { (yyval.expr) = (yyvsp[(1) - (1)].expr); }
     break;
 
   case 50:
-#line 780 "parser.y"
+#line 800 "parser.y"
     {
 		GnmExpr *tmp = fold_negative_constant ((yyvsp[(2) - (2)].expr));
 		if (!tmp) { YYERROR; }
@@ -2363,7 +2383,7 @@ yyreduce:
     break;
 
   case 51:
-#line 785 "parser.y"
+#line 805 "parser.y"
     {
 		GnmExpr *tmp = fold_positive_constant ((yyvsp[(2) - (2)].expr));
 		if (!tmp) { YYERROR; }
@@ -2372,17 +2392,17 @@ yyreduce:
     break;
 
   case 52:
-#line 790 "parser.y"
+#line 810 "parser.y"
     { (yyval.expr) = parse_string_as_value ((yyvsp[(1) - (1)].expr)); }
     break;
 
   case 53:
-#line 794 "parser.y"
+#line 814 "parser.y"
     { (yyval.list) = NULL; }
     break;
 
   case 54:
-#line 795 "parser.y"
+#line 815 "parser.y"
     {
 		unregister_allocation ((yyvsp[(1) - (1)].expr));
 		(yyval.list) = g_slist_prepend (NULL, (yyvsp[(1) - (1)].expr));
@@ -2391,7 +2411,7 @@ yyreduce:
     break;
 
   case 55:
-#line 800 "parser.y"
+#line 820 "parser.y"
     {
 		unregister_allocation ((yyvsp[(3) - (3)].list));
 		unregister_allocation ((yyvsp[(1) - (3)].expr));
@@ -2401,7 +2421,7 @@ yyreduce:
     break;
 
   case 56:
-#line 808 "parser.y"
+#line 828 "parser.y"
     {
 		unregister_allocation ((yyvsp[(1) - (1)].list));
 		(yyval.list) = g_slist_prepend (NULL, (yyvsp[(1) - (1)].list));
@@ -2410,7 +2430,7 @@ yyreduce:
     break;
 
   case 57:
-#line 813 "parser.y"
+#line 833 "parser.y"
     {
 		unregister_allocation ((yyvsp[(3) - (3)].list));
 		unregister_allocation ((yyvsp[(1) - (3)].list));
@@ -2421,7 +2441,7 @@ yyreduce:
 
 
 /* Line 1267 of yacc.c.  */
-#line 2425 "parser.c"
+#line 2445 "parser.c"
       default: break;
     }
   YY_SYMBOL_PRINT ("-> $$ =", yyr1[yyn], &yyval, &yyloc);
@@ -2635,7 +2655,7 @@ yyreturn:
 }
 
 
-#line 821 "parser.y"
+#line 841 "parser.y"
 
 
 static char const *
@@ -2896,7 +2916,7 @@ yylex (void)
 
 	if (start != (end = state->convs->input.range_ref (&ref, start, state->pos, state->convs))) {
 		state->ptr = end;
-		if (state->force_absolute_col_references) {
+		if (state->flags & GNM_EXPR_PARSE_FORCE_ABSOLUTE_REFERENCES) {
 			if (ref.a.col_relative) {
 				ref.a.col += state->pos->eval.col;
 				ref.a.col_relative = FALSE;
@@ -2905,8 +2925,6 @@ yylex (void)
 				ref.b.col += state->pos->eval.col;
 				ref.b.col_relative = FALSE;
 			}
-		}
-		if (state->force_absolute_row_references) {
 			if (ref.a.row_relative) {
 				ref.a.row += state->pos->eval.row;
 				ref.a.row_relative = FALSE;
@@ -2915,9 +2933,26 @@ yylex (void)
 				ref.b.row += state->pos->eval.row;
 				ref.b.row_relative = FALSE;
 			}
+		} else if (state->flags & GNM_EXPR_PARSE_FORCE_RELATIVE_REFERENCES) {
+			if (!ref.a.col_relative) {
+				ref.a.col -= state->pos->eval.col;
+				ref.a.col_relative = TRUE;
+			}
+			if (!ref.b.col_relative) {
+				ref.b.col -= state->pos->eval.col;
+				ref.b.col_relative = TRUE;
+			}
+			if (!ref.a.row_relative) {
+				ref.a.row -= state->pos->eval.row;
+				ref.a.row_relative = TRUE;
+			}
+			if (!ref.b.row_relative) {
+				ref.b.row -= state->pos->eval.row;
+				ref.b.row_relative = TRUE;
+			}
 		}
 
-		if (ref.a.sheet == NULL && state->force_explicit_sheet_references) {
+		if (ref.a.sheet == NULL && (state->flags & GNM_EXPR_PARSE_FORCE_EXPLICIT_SHEET_REFERENCES)) {
 			ref.a.sheet = state->pos->sheet;
 			if (ref.a.sheet == NULL) {
 				report_err (state, g_error_new (1, PERR_SHEET_IS_REQUIRED,
@@ -3170,11 +3205,7 @@ gnm_expr_parse_str (char const *str, GnmParsePos const *pp,
 	pstate.start = pstate.ptr = str;
 	pstate.pos   = pp;
 
-	pstate.force_absolute_col_references		= flags & GNM_EXPR_PARSE_FORCE_ABSOLUTE_COL_REFERENCES;
-	pstate.force_absolute_row_references		= flags & GNM_EXPR_PARSE_FORCE_ABSOLUTE_ROW_REFERENCES;
-	pstate.force_explicit_sheet_references		= flags & GNM_EXPR_PARSE_FORCE_EXPLICIT_SHEET_REFERENCES;
-	pstate.unknown_names_are_strings		= flags & GNM_EXPR_PARSE_UNKNOWN_NAMES_ARE_STRINGS;
-	pstate.unknown_names_are_invalid		= flags & GNM_EXPR_PARSE_UNKNOWN_NAMES_ARE_INVALID;
+	pstate.flags		= flags;
 	pstate.convs                                    =
 		(NULL != convs) ? convs : ((NULL != pp->sheet) ? pp->sheet->convs : gnm_conventions_default);
 
