@@ -580,7 +580,6 @@ static struct {
 	{ N_("cell"),  render_cell  , NULL},
 	{ NULL },
 };
-
 /*
  * Renders an opcode.  The opcodes can take an argument by adding trailing ':'
  * to the opcode and then a number format code
@@ -613,6 +612,34 @@ render_opcode (GString *target, char /* non-const */ *opcode,
 	g_free (opcode_trans);
 }
 
+#if 0
+3.3.1.36 evenHeader (Even Page Header)
+
+          &P              Current page number
+          &N              Page count
+          &D              Current date
+          &T              Current time
+          &A              Sheet name (BIFF5-BIFF8)
+          &F              File name without path
+          &Z              File path without file name (BIFF8X)
+          &G              Picture (BIFF8X)
+          &B              Bold on/off (BIFF2-BIFF4)
+          &I              Italic on/off (BIFF2-BIFF4)
+          &U              Underlining on/off
+          &E              Double underlining on/off (BIFF5-BIFF8)
+          &S              Strikeout on/off
+          &X              Superscript on/off (BIFF5-BIFF8)
+          &Y              Subscript on/off (BIFF5-BIFF8)
+     &"<fontname>"        Set new font <fontname>
+&"<fontname>,<fontstyle>" Set new font with specified style <fontstyle>. The style <fontstyle> is in most cases
+                          one of “Regular”, “Bold”, “Italic”, or “Bold Italic”. But this setting is dependent on the
+                          used font, it may differ (localised style names, or “Standard”, “Oblique”, ...). (BIFF5-
+                          BIFF8)
+     &<fontheight>        Set font height in points (<fontheight> is a decimal value). If this command is followed
+                          by a plain number to be printed in the header, it will be separated from the font height
+                          with a space character.
+#endif
+
 char *
 hf_format_render (char const *format, HFRenderInfo *info, HFRenderType render_type)
 {
@@ -1082,21 +1109,36 @@ print_info_get_page_setup (PrintInformation *pi)
 		return NULL;
 }
 
+/**
+ * print_info_set_page_setup :
+ * @pi : #PrintInformation
+ * @page_setup : #GtkPageSetup
+ *
+ * Absorb a ref to @page_setup.
+ *
+ * WHY WHY WHY
+ * 1) The life cycle in here is a tad odd, the load_defaults does nothing for the
+ * case of an existing page_setup, and seems like it should be ignored for the
+ * case of a new one.
+ *
+ * 2) Why not copy the page_setup in here and make the arg const ?
+ **/
 void
 print_info_set_page_setup (PrintInformation *pi, GtkPageSetup *page_setup)
 {
-	double header, footer, left, right;
-
 	g_return_if_fail (pi != NULL);
+
 	print_info_load_defaults (pi);
 
 	if (pi->page_setup) {
+		double header, footer, left, right;
+		print_info_get_margins (pi,
+			&header, &footer, &left, &right, NULL, NULL);
 		g_object_unref (pi->page_setup);
-		print_info_get_margins (pi, &header, &footer, &left, &right,
-					NULL, NULL);
 		pi->page_setup = page_setup;
 		print_info_set_margins (pi, header, footer, left, right);
-	} else pi->page_setup = page_setup;
+	} else
+		pi->page_setup = page_setup;
 }
 
 GtkPageOrientation
@@ -1165,7 +1207,8 @@ gnm_page_breaks_new (int len, gboolean is_vert)
 {
 	GnmPageBreaks *res = g_new (GnmPageBreaks, 1);
 
-	if (len < 0 || len > colrow_max (is_vert))
+#warning "We cannot use NULL here."
+	if (len < 0 || len > colrow_max (is_vert, NULL))
 		len = 0;
 	res->is_vert = is_vert;
 	res->details = g_array_sized_new (FALSE, FALSE,
