@@ -559,7 +559,7 @@ typedef enum {
 	STF_CELL_EOF,
 	STF_CELL_EOL,
 	STF_CELL_FIELD_NO_SEP,
-	STF_CELL_FIELD_SEP,
+	STF_CELL_FIELD_SEP
 } StfParseCellRes;
 
 static StfParseCellRes
@@ -817,17 +817,23 @@ stf_parse_general (StfParseOptions_t *parseoptions,
 	GPtrArray *lines;
 	Source_t src;
 	int row;
+	char const *valid_end = data_end;
 
 	g_return_val_if_fail (parseoptions != NULL, NULL);
 	g_return_val_if_fail (data != NULL, NULL);
 	g_return_val_if_fail (data_end != NULL, NULL);
 	g_return_val_if_fail (stf_parse_options_valid (parseoptions), NULL);
-	g_return_val_if_fail (g_utf8_validate (data, -1, NULL), NULL);
+	g_return_val_if_fail (g_utf8_validate (data, data_end-data, &valid_end), NULL);
 
 	src.chunk = lines_chunk;
 	src.position = data;
 	row = 0;
 
+	if ((data_end-data >= 3) && !strncmp(src.position, "\xEF\xBB\xBF", 3)) {
+		/* Skip over byte-order mark */
+		src.position += 3;
+	}
+
 	lines = g_ptr_array_new ();
 	while (*src.position != '\0' && src.position < data_end) {
 		GPtrArray *line;
@@ -840,7 +846,7 @@ stf_parse_general (StfParseOptions_t *parseoptions,
 		if (parseoptions->parsetype != PARSE_TYPE_CSV)
 			src.position += compare_terminator (src.position, parseoptions);
 
-		if (++row == SHEET_MAX_ROWS)
+		if (++row == gnm_sheet_get_max_rows (NULL))
 			break;
 	}
 
@@ -1203,7 +1209,7 @@ stf_parse_sheet (StfParseOptions_t *parseoptions,
 			if (parseoptions->col_import_array == NULL ||
 			    parseoptions->col_import_array_len <= lcol ||
 			    parseoptions->col_import_array[lcol]) {
-				if (col >= SHEET_MAX_COLS) {
+				if (col >= gnm_sheet_get_max_cols (sheet)) {
 					if (!parseoptions->cols_exceeded) {
 						g_warning (_("There are more columns of data than "
 							     "there is room for in the sheet.  Extra "
