@@ -1,12 +1,14 @@
 /* vim: set sw=8: -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-#ifndef GNM_VALUE_H
-#define GNM_VALUE_H
+#ifndef _GNM_VALUE_H_
+# define _GNM_VALUE_H_
 
 #include "gnumeric.h"
 #include "position.h"
 #include "numbers.h"
 #include <glib-object.h>
 
+G_BEGIN_DECLS
+
 typedef enum {
 	/* Use magic values to act as a signature
 	 * DO NOT CHANGE THESE NUMBERS
@@ -117,6 +119,7 @@ GnmValue *value_new_cellrange        (GnmCellRef const *a, GnmCellRef const *b,
 				      int eval_col, int eval_row);
 GnmValue *value_new_cellrange_r      (Sheet *sheet, GnmRange const *r);
 GnmValue *value_new_cellrange_str    (Sheet *sheet, char const *str);
+GnmValue *value_new_cellrange_parsepos_str (GnmParsePos *pp, char const *str);
 GnmValue *value_new_array            (guint cols, guint rows);
 GnmValue *value_new_array_empty      (guint cols, guint rows);
 GnmValue *value_new_array_non_init   (guint cols, guint rows);
@@ -183,7 +186,8 @@ extern GnmValueErr const value_terminate_err;
 void value_array_set       (GnmValue *array, int col, int row, GnmValue *v);
 
 /* FIXME: this stuff below ought to go elsewhere.  */
-typedef gboolean (*GnmCriteriaFunc) (GnmValue const *x, GnmValue const *y);
+typedef gboolean (*GnmCriteriaFunc) (GnmValue const *x, GnmValue const *y,
+				     GODateConventions const *date_conv);
 typedef struct {
         GnmCriteriaFunc fun;
         GnmValue  *x;
@@ -212,4 +216,6 @@ int     find_column_of_field	(GnmEvalPos const *ep,
 void value_init     (void);
 void value_shutdown (void);
 
-#endif /* GNM_VALUE_H */
+G_END_DECLS
+
+#endif /* _GNM_VALUE_H_ */
