@@ -67,6 +67,7 @@
 #include <goffice/graph/gog-data-allocator.h>
 #include <goffice/graph/gog-data-set.h>
 #include <goffice/utils/go-file.h>
+#include <goffice/utils/go-locale.h>
 #include <goffice/utils/go-glib-extras.h>
 
 #include "widgets/widget-editable-label.h"
@@ -122,9 +123,10 @@ static GNM_ACTION_DEF (cb_file_print_area_clear)
 {
 	GnmParsePos pp;
 	GnmRange r;
+	Sheet *sheet = wbcg_cur_sheet (wbcg);
 
 	range_init_full_sheet (&r);
-	parse_pos_init_sheet (&pp, wbcg_cur_sheet (wbcg));
+	parse_pos_init_sheet (&pp, sheet);
 	cmd_define_name	(WORKBOOK_CONTROL (wbcg), "Print_Area", &pp,
 			 gnm_expr_top_new_constant
 			 (value_new_cellrange_r (NULL, &r)),
@@ -576,13 +578,13 @@ insert_date_time_common (WBCGtk *wbcg, int what)
 		WorkbookControl *wbc = WORKBOOK_CONTROL (wbcg);
 		SheetView *sv = wb_control_cur_sheet_view (wbc);
 		Sheet *sheet = sv_sheet (sv);
-		GnmCell const *cell = sheet_cell_get (sheet,
-						      sv->edit_pos.col,
-						      sv->edit_pos.row);
+		GnmCell const *cell = sheet_cell_fetch (sheet,
+							sv->edit_pos.col,
+							sv->edit_pos.row);
 		GODateConventions const *date_conv =
 			workbook_date_conv (sheet->workbook);
-		GnmValue *v = value_new_int (
-			datetime_timet_to_serial (time (NULL), date_conv));
+		GnmValue *v = value_new_float
+			(datetime_timet_to_serial_raw (time (NULL), date_conv));
 		GOFormat *vfmt;
 		char *txt;
 
@@ -726,7 +728,9 @@ static GNM_ACTION_DEF (cb_tools_desc_statistics) { dialog_descriptive_stat_tool
 static GNM_ACTION_DEF (cb_tools_exp_smoothing)	{ dialog_exp_smoothing_tool (wbcg, wbcg_cur_sheet (wbcg)); }
 static GNM_ACTION_DEF (cb_tools_average)	{ dialog_average_tool (wbcg, wbcg_cur_sheet (wbcg)); }
 static GNM_ACTION_DEF (cb_tools_fourier)	{ dialog_fourier_tool (wbcg, wbcg_cur_sheet (wbcg)); }
+static GNM_ACTION_DEF (cb_tools_frequency)	{ dialog_frequency_tool (wbcg, wbcg_cur_sheet (wbcg)); }
 static GNM_ACTION_DEF (cb_tools_histogram)	{ dialog_histogram_tool (wbcg, wbcg_cur_sheet (wbcg)); }
+static GNM_ACTION_DEF (cb_tools_kaplan_meier)	{ dialog_kaplan_meier_tool (wbcg, wbcg_cur_sheet (wbcg)); }
 static GNM_ACTION_DEF (cb_tools_ranking)	{ dialog_ranking_tool (wbcg, wbcg_cur_sheet (wbcg)); }
 static GNM_ACTION_DEF (cb_tools_regression)	{ dialog_regression_tool (wbcg, wbcg_cur_sheet (wbcg)); }
 static GNM_ACTION_DEF (cb_tools_sampling)	{ dialog_sampling_tool (wbcg, wbcg_cur_sheet (wbcg)); }
@@ -869,22 +873,27 @@ static GNM_ACTION_DEF (cb_data_ungroup)		{ group_ungroup_colrow (wbcg, FALSE); }
 static GNM_ACTION_DEF (cb_help_docs)
 {
 	char   *argv[] = { NULL, NULL, NULL };
-	GError *error = NULL;
+	GError *err = NULL;
 
 #ifndef G_OS_WIN32
 	argv[0] = (char *)"yelp";
 	argv[1] = (char *)"ghelp:gnumeric";
 	g_spawn_async (NULL, argv, NULL, G_SPAWN_SEARCH_PATH,
-		NULL, NULL, NULL, &error);
+		NULL, NULL, NULL, &err);
 #else
-	/* TODO : handle translations when we generate them */
+	/* TODO : Should really start in same directory as the gspawn-* helpers
+	 * are installed in case they are not in the path */
 	argv[0] = (char *)"hh";
 	argv[1] = g_build_filename (gnm_sys_data_dir (), "doc", "C",
 			"gnumeric.chm", NULL);
 	g_spawn_async (NULL, argv, NULL, G_SPAWN_SEARCH_PATH,
-		NULL, NULL, NULL, &error);
+		NULL, NULL, NULL, &err);
 	g_free (argv[1]);
 #endif
+	if (NULL != err) {
+		go_cmd_context_error (GO_CMD_CONTEXT (wbcg), err);
+		g_error_free (err);
+	}
 }
 
 static GNM_ACTION_DEF (cb_help_web)
@@ -956,7 +965,7 @@ static GNM_ACTION_DEF (cb_insert_image)
 			guint8 const *data = gsf_input_read (input, len, NULL);
 			SheetObjectImage *soi = g_object_new (SHEET_OBJECT_IMAGE_TYPE, NULL);
 			sheet_object_image_set_image (soi, "", (guint8 *)data, len, TRUE);
-			scg_mode_create_object (wbcg_cur_scg (wbcg), SHEET_OBJECT (soi));
+			wbcg_insert_object (wbcg, SHEET_OBJECT (soi));
 			g_object_unref (input);
 		} else
 			go_cmd_context_error (GO_CMD_CONTEXT (wbcg), err);
@@ -1031,8 +1040,7 @@ static GNM_ACTION_DEF (cb_sort_descending) { sort_by_rows (wbcg, TRUE); }
 static void
 cb_add_graph (GogGraph *graph, gpointer wbcg)
 {
-	SheetControlGUI *scg = wbcg_cur_scg (WBC_GTK (wbcg));
-	scg_mode_create_object (scg, sheet_object_graph_new (graph));
+	wbcg_insert_object (WBC_GTK (wbcg), sheet_object_graph_new (graph));
 }
 
 static GNM_ACTION_DEF (cb_launch_chart_guru)
@@ -1048,17 +1056,11 @@ create_object (WBCGtk *wbcg, GType t,
 	       char const *first_property_name,
 	       ...)
 {
-	SheetControlGUI *scg = wbcg_cur_scg (wbcg);
-	Sheet *sheet = scg_sheet (scg);
 	va_list	args;
-
 	va_start (args, first_property_name);
-	scg_mode_create_object (scg, (SheetObject *)
+	wbcg_insert_object (wbcg, (SheetObject *)
 		g_object_new_valist (t, first_property_name, args));
 	va_end (args);
-
-	workbook_recalc (sheet->workbook);
-	sheet_update (sheet);
 }
 
 static GNM_ACTION_DEF (cmd_create_label)
@@ -1372,10 +1374,98 @@ modify_format (WBCGtk *wbcg,
 	}
 }
 
+static GnmValue *
+cb_calc_decs (GnmCellIter const *iter, gpointer user)
+{
+	int *pdecs = user;
+	int decs = 0;
+	GnmCell *cell = iter->cell;
+	char *text;
+	const char *p;
+	GString const *dec = go_locale_get_decimal ();
+
+	if (!cell || !cell->value || !VALUE_IS_NUMBER (cell->value))
+		return NULL;
+
+	/*
+	 * If we are displaying an equation, we don't want to look into
+	 * the rendered text.
+	 */
+	if (gnm_cell_has_expr (cell) && cell->base.sheet->display_formulas)
+		return NULL;
+
+	text = gnm_cell_get_rendered_text (cell);
+	p = strstr (text, dec->str);
+	if (p) {
+		p += dec->len;
+
+		while (g_ascii_isdigit (*p))
+			decs++, p++;
+	}
+
+	*pdecs = MAX (*pdecs, decs);
+
+	g_free (text);
+
+	return NULL;
+}
+
+static void
+inc_dec (WBCGtk *wbcg,
+	 int dir,
+	 GOFormat *(*format_modify_fn) (GOFormat const *format),
+	 char const *descriptor)
+{
+	WorkbookControl *wbc = WORKBOOK_CONTROL (wbcg);
+	WorkbookView *wbv = wb_control_view (wbc);
+	GOFormat const *fmt = gnm_style_get_format (wbv->current_style);
+	SheetView *sv;
+	GOFormat *new_fmt;
+	GnmStyle *style;
+	GSList *l;
+	int decs = -2;
+	GString *new_fmt_str;
+
+	if (!go_format_is_general (fmt)) {
+		modify_format (wbcg, format_modify_fn, descriptor);
+		return;
+	}
+
+	sv = wb_view_cur_sheet_view (wbv);
+	if (!sv)
+		return;
+
+	for (l = sv->selections; l ; l = l->next) {
+		GnmRange const *r = l->data;
+		sheet_foreach_cell_in_range (sv_sheet (sv),
+					     CELL_ITER_IGNORE_BLANK |
+					     CELL_ITER_IGNORE_HIDDEN,
+					     r->start.col, r->start.row,
+					     r->end.col, r->end.row,
+					     cb_calc_decs,
+					     &decs);
+	}
+
+	new_fmt_str = g_string_new ("0");
+	if (decs + dir > 0) {
+		g_string_append_c (new_fmt_str, '.');
+		go_string_append_c_n (new_fmt_str, '0', decs + dir);
+	}
+	new_fmt = go_format_new_from_XL (new_fmt_str->str);
+	g_string_free (new_fmt_str, TRUE);
+
+	style = gnm_style_new ();
+	gnm_style_set_format (style, new_fmt);
+	cmd_selection_format (wbc, style, NULL, descriptor);
+	go_format_unref (new_fmt);
+}
+
 static GNM_ACTION_DEF (cb_format_inc_precision)
-	{ modify_format (wbcg, &go_format_inc_precision, _("Increase precision")); }
+	{ inc_dec (wbcg, 1,
+		   &go_format_inc_precision, _("Increase precision")); }
 static GNM_ACTION_DEF (cb_format_dec_precision)
-	{ modify_format (wbcg, &go_format_dec_precision, _("Decrease precision")); }
+	{ inc_dec (wbcg, -1,
+		   &go_format_dec_precision, _("Decrease precision")); }
 static GNM_ACTION_DEF (cb_format_with_thousands)
 	{ modify_format (wbcg, &go_format_toggle_1000sep, _("Toggle thousands separator")); }
 
@@ -1465,6 +1555,7 @@ static GtkActionEntry const permanent_actions[] = {
 		{ "MenuToolStatisticalAnalysis",	NULL,	N_("Statistical Anal_ysis") },
 		{ "MenuANOVA",	NULL,	N_("_ANOVA") },
 		{ "MenuToolForecast",	NULL,	N_("F_orecast") },
+		{ "MenuToolFrequencies",	NULL,	N_("Fre_quency Tables") },
 		{ "MenuToolTTest",	NULL,	N_("Two _Means") },
 	{ "MenuData",		NULL, N_("_Data") },
 		{ "MenuFilter",		NULL,	N_("_Filter") },
@@ -1593,7 +1684,10 @@ static GtkActionEntry const actions[] = {
 	{ "EditDeleteCells", NULL, N_("C_ells..."),
 		  "<control>minus", N_("Delete the selected cells, shifting others into their place"),
 		  G_CALLBACK (cb_edit_delete_cells) },
-
+	/* A duplicate that should not go into the menus, used only for the accelerator */
+	{ "EditDeleteCellsXL", NULL, N_("C_ells..."),
+		  "<control>KP_Subtract", N_("Delete the selected cells, shifting others into their place"),
+		  G_CALLBACK (cb_edit_delete_cells) },
 
 /* Edit -> Select */
 
@@ -1714,6 +1808,10 @@ static GtkActionEntry const actions[] = {
 	{ "InsertCells", NULL, N_("C_ells..."),
 		"<control>plus", N_("Insert new cells"),
 		G_CALLBACK (cb_insert_cells) },
+	/* A duplicate that should not go into the menus, used only for the accelerator */
+	{ "InsertCellsXL", NULL, N_("C_ells..."),
+		"<control>KP_Add", N_("Insert new cells"),
+		G_CALLBACK (cb_insert_cells) },
 	{ "InsertColumns", "Gnumeric_ColumnAdd", N_("_Columns"),
 		NULL, N_("Insert new columns"),
 		G_CALLBACK (cb_insert_cols) },
@@ -1853,6 +1951,14 @@ static GtkActionEntry const actions[] = {
 		NULL, N_("Moving average..."),
 		G_CALLBACK (cb_tools_average) },
 
+/* Tools -> Frequency Tables */
+	{ "ToolsFrequency", NULL, N_("Fre_quency Tables..."),
+		NULL, N_("Frequency tables for non-numeric data"),
+		G_CALLBACK (cb_tools_frequency) },
+	{ "ToolsHistogram", NULL, N_("_Histogram..."),
+		NULL, N_("Various frequency tables for numeric data"),
+		G_CALLBACK (cb_tools_histogram) },
+	
 /* Tools -> Two Means */
 	{ "ToolTTestPaired", NULL, N_("_Paired Samples: T-Test..."),
 		NULL, N_("Comparing two population means for two paired samples: t-test..."),
@@ -1883,9 +1989,6 @@ static GtkActionEntry const actions[] = {
 	{ "ToolsFourier", NULL, N_("_Fourier Analysis..."),
 		NULL, N_("Fourier Analysis"),
 		G_CALLBACK (cb_tools_fourier) },
-	{ "ToolsHistogram", NULL, N_("_Histogram..."),
-		NULL, N_("Various frequency tables"),
-		G_CALLBACK (cb_tools_histogram) },
 	{ "ToolsRanking", NULL, N_("Ranks And _Percentiles..."),
 		NULL, N_("Ranks, placements and percentiles"),
 		G_CALLBACK (cb_tools_ranking) },
@@ -1898,6 +2001,9 @@ static GtkActionEntry const actions[] = {
 	{ "ToolsFTest", NULL, N_("_Two Variances: FTest..."),
 		NULL, N_("Comparing two population variances"),
 		G_CALLBACK (cb_tools_ftest) },
+	{ "ToolsKaplanMeier", NULL, N_("_Kaplan-Meier Estimates..."),
+		NULL, N_("Creation of Kaplan-Meier Survival Curves"),
+		G_CALLBACK (cb_tools_kaplan_meier) },
 
 /* Data */
 	{ "DataSort", GTK_STOCK_SORT_ASCENDING, N_("_Sort..."),
