@@ -3,7 +3,7 @@
 /*
  * gnm-cell-combo-foo-view.c: A foocanvas object for an in-cell combo-box
  *
- * Copyright (C) 2006 Jody Goldberg (jody@gnome.org)
+ * Copyright (C) 2006-2007 Jody Goldberg (jody@gnome.org)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2 of the GNU General Public
@@ -24,10 +24,10 @@
 #include "gnm-cell-combo-foo-view.h"
 #include "gnm-cell-combo-foo-view-impl.h"
 
+#include "wbc-gtk.h"
 #include "sheet.h"
 #include "sheet-control-gui.h"
-#include "gnumeric-pane.h"
-#include "gnumeric-canvas.h"
+#include "gnm-pane-impl.h"
 #include "sheet-object-impl.h"
 
 #include <goffice/cut-n-paste/foocanvas/foo-canvas-widget.h>
@@ -53,11 +53,11 @@ ccombo_activate (GtkWidget *popup, GtkTreeView *list)
 {
 	SheetObjectView		*sov    = g_object_get_data (G_OBJECT (list), SOV_ID);
 	FooCanvasItem		*view   = FOO_CANVAS_ITEM (sov);
-	GnmPane			*pane   = GNM_CANVAS (view->canvas)->pane;
+	GnmPane			*pane   = GNM_PANE (view->canvas);
 	GnmCComboFooViewIface	*iface  = GNM_CCOMBO_FOO_VIEW_GET_CLASS (sov);
 
 	(iface->activate) (sheet_object_view_get_so (sov), popup, list,
-			   scg_wbcg (pane->gcanvas->simple.scg));
+			   scg_wbcg (pane->simple.scg));
 	ccombo_popup_destroy (popup, GTK_WIDGET (list));
 }
 
@@ -157,6 +157,7 @@ cb_ccombo_key_press (GtkWidget *popup, GdkEventKey *event, GtkWidget *list)
 	case GDK_KP_Down :
 	case GDK_Down :
 	case GDK_KP_Up :
+		/* fallthrough */
 	case GDK_Up :
 		if (!(event->state & GDK_MOD1_MASK))
 			return FALSE;
@@ -238,42 +239,33 @@ cb_ccombo_button_release (GtkWidget *popup, GdkEventButton *event,
 		    ccombo_activate (popup, list);
 		    return TRUE;
 	    }
-	    g_signal_handlers_disconnect_by_func (popup, 
+	    g_signal_handlers_disconnect_by_func (popup,
 			G_CALLBACK (cb_ccombo_popup_motion), list);
 	    ccombo_autoscroll_set (G_OBJECT (list), 0);
 	}
 	return FALSE;
 }
 
-#if 0
-static void
-cb_focus_changed (GtkWindow *toplevel,
-		  G_GNUC_UNUSED GParamSpec *pspec,
-		  SheetObjectView *sov)
-{
-	g_warning (gtk_window_has_toplevel_focus (toplevel) ? "focus" : "no focus");
-}
-#endif
-
 static void
 cb_ccombo_button_pressed (G_GNUC_UNUSED GtkButton *button,
 			  SheetObjectView *sov)
 {
-	gnm_cell_combo_foo_view_popdown (sov);
+	gnm_cell_combo_foo_view_popdown (sov, GDK_CURRENT_TIME);
 }
 
 /**
  * gnm_cell_combo_foo_view_popdown:
  * @sov : #SheetObjectView
+ * @activate_time : event time
  *
- * Open the popup window
+ * Open the popup window associated with @sov
  **/
 void
-gnm_cell_combo_foo_view_popdown (SheetObjectView *sov)
+gnm_cell_combo_foo_view_popdown (SheetObjectView *sov, guint32 activate_time)
 {
 	FooCanvasItem	   *view   = FOO_CANVAS_ITEM (sov);
-	GnmPane		   *pane   = GNM_CANVAS (view->canvas)->pane;
-	SheetControlGUI	   *scg    = pane->gcanvas->simple.scg;
+	GnmPane		   *pane   = GNM_PANE (view->canvas);
+	SheetControlGUI	   *scg    = pane->simple.scg;
 	SheetObject	   *so     = sheet_object_view_get_so (sov);
 	Sheet const	   *sheet  = sheet_object_get_sheet (so);
 	GtkWidget *frame,  *popup, *list, *container;
@@ -282,8 +274,17 @@ gnm_cell_combo_foo_view_popdown (SheetObjectView *sov)
 	GtkTreeViewColumn *column;
 	GtkTreePath	  *clip = NULL, *select = NULL;
 	GtkRequisition	req;
+	GtkWindow *toplevel = wbcg_toplevel (scg_wbcg (scg));
 
 	popup = gtk_window_new (GTK_WINDOW_POPUP);
+	gtk_window_set_type_hint (GTK_WINDOW (popup), GDK_WINDOW_TYPE_HINT_COMBO);
+	gtk_window_group_add_window (gtk_window_get_group (toplevel), GTK_WINDOW (popup));
+	go_gtk_window_set_transient (toplevel, GTK_WINDOW (popup));
+	gtk_window_set_resizable (GTK_WINDOW (popup), FALSE);
+	gtk_window_set_decorated (GTK_WINDOW (popup), FALSE);
+	gtk_window_set_screen (GTK_WINDOW (popup),
+		gtk_widget_get_screen (GTK_WIDGET (toplevel)));
+
 	model = ccombo_fill_model (GNM_CCOMBO_FOO_VIEW (sov), so, &clip, &select);
 	column = gtk_tree_view_column_new_with_attributes ("ID",
 		gtk_cell_renderer_text_new (), "text", 0,
@@ -294,18 +295,13 @@ gnm_cell_combo_foo_view_popdown (SheetObjectView *sov)
 	gtk_tree_view_append_column (GTK_TREE_VIEW (list), column);
 	gtk_widget_size_request (GTK_WIDGET (list), &req);
 	g_object_set_data (G_OBJECT (list), SOV_ID, sov);
-#if 0
-	id = g_signal_connect (G_OBJECT (wbcg_toplevel (scg_wbcg (scg))),
-		"notify::has-toplevel-focus",
-		G_CALLBACK (cb_focus_changed), sov);
-#endif
 
 	frame = gtk_frame_new (NULL);
 	gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_OUT);
 
 #if 0
 	range_dump (&so->anchor.cell_bound, "");
-	fprintf (stderr, " : so = %p, view = %p\n", so, view);
+	g_printerr (" : so = %p, view = %p\n", so, view);
 #endif
 	if (clip != NULL) {
 		GdkRectangle  rect;
@@ -328,21 +324,20 @@ gnm_cell_combo_foo_view_popdown (SheetObjectView *sov)
 	gtk_container_add (GTK_CONTAINER (frame), container);
 
 	/* do the popup */
-	gtk_window_set_decorated (GTK_WINDOW (popup), FALSE);
-	gdk_window_get_origin (GTK_WIDGET (pane->gcanvas)->window,
+	gdk_window_get_origin (GTK_WIDGET (pane)->window,
 		&root_x, &root_y);
 	if (sheet->text_is_rtl) {
-		root_x += GTK_WIDGET(pane->gcanvas)->allocation.width;
+		root_x += GTK_WIDGET (pane)->allocation.width;
 		root_x -= scg_colrow_distance_get (scg, TRUE,
-			pane->gcanvas->first.col,
+			pane->first.col,
 			so->anchor.cell_bound.start.col + 1);
 	} else
 		root_x += scg_colrow_distance_get (scg, TRUE,
-			pane->gcanvas->first.col,
+			pane->first.col,
 			so->anchor.cell_bound.start.col);
 	gtk_window_move (GTK_WINDOW (popup), root_x,
 		root_y + scg_colrow_distance_get (scg, FALSE,
-			pane->gcanvas->first.row,
+			pane->first.row,
 			so->anchor.cell_bound.start.row + 1));
 
 	gtk_container_add (GTK_CONTAINER (popup), frame);
@@ -371,15 +366,22 @@ gnm_cell_combo_foo_view_popdown (SheetObjectView *sov)
 		gtk_tree_path_free (select);
 	}
 
+	gtk_widget_grab_focus (popup);
 	gtk_widget_grab_focus (GTK_WIDGET (list));
 	ccombo_focus_change (GTK_WIDGET (list), TRUE);
 
-	gtk_grab_add (popup);
-	gdk_pointer_grab (popup->window, TRUE,
+	if (0 == gdk_pointer_grab (popup->window, TRUE,
 		GDK_BUTTON_PRESS_MASK |
 		GDK_BUTTON_RELEASE_MASK |
 		GDK_POINTER_MOTION_MASK,
-		NULL, NULL, GDK_CURRENT_TIME);
+		NULL, NULL, activate_time)) {
+		if (0 ==  gdk_keyboard_grab (popup->window, TRUE, activate_time)) {
+			gtk_grab_add (popup);
+		} else {
+			gdk_display_pointer_ungrab (
+				gdk_drawable_get_display (popup->window), activate_time);
+		}
+	}
 }
 
 /**
@@ -394,9 +396,9 @@ SheetObjectView *
 gnm_cell_combo_foo_view_new (SheetObject *so, GType type,
 			     SheetObjectViewContainer *container)
 {
-	GnmCanvas *gcanvas = ((GnmPane *)container)->gcanvas;
+	GnmPane *pane = GNM_PANE (container);
 	GtkWidget *view_widget = gtk_button_new ();
-	FooCanvasItem *ccombo = foo_canvas_item_new (gcanvas->object_views, type,
+	FooCanvasItem *ccombo = foo_canvas_item_new (pane->object_views, type,
 		"widget",	view_widget,
 		"size_pixels",	FALSE,
 		NULL);
