@@ -28,11 +28,11 @@
 G_BEGIN_DECLS
 
 #define GNUMERIC_TYPE_CELL_RENDERER_TOGGLE		(gnumeric_cell_renderer_toggle_get_type ())
-#define GNUMERIC_CELL_RENDERER_TOGGLE(obj)		(GTK_CHECK_CAST ((obj), GNUMERIC_TYPE_CELL_RENDERER_TOGGLE, GnumericCellRendererToggle))
-#define GNUMERIC_CELL_RENDERER_TOGGLE_CLASS(klass)	(GTK_CHECK_CLASS_CAST ((klass), GNUMERIC_TYPE_CELL_RENDERER_TOGGLE, GnumericCellRendererToggleClass))
-#define GNUMERIC_IS_CELL_RENDERER_TOGGLE(obj)		(GTK_CHECK_TYPE ((obj), GNUMERIC_TYPE_CELL_RENDERER_TOGGLE))
-#define GNUMERIC_IS_CELL_RENDERER_TOGGLE_CLASS(klass)	(GTK_CHECK_CLASS_TYPE ((klass), GNUMERIC_TYPE_CELL_RENDERER_TOGGLE))
-#define GNUMERIC_CELL_RENDERER_TOGGLE_GET_CLASS(obj)   (GTK_CHECK_GET_CLASS ((obj), GNUMERIC_TYPE_CELL_RENDERER_TOGGLE, GnumericCellRendererToggleClass))
+#define GNUMERIC_CELL_RENDERER_TOGGLE(o)		(G_TYPE_CHECK_INSTANCE_CAST ((o), GNUMERIC_TYPE_CELL_RENDERER_TOGGLE, GnumericCellRendererToggle))
+#define GNUMERIC_CELL_RENDERER_TOGGLE_CLASS(k)		(G_TYPE_CHECK_CLASS_CAST ((k), GNUMERIC_TYPE_CELL_RENDERER_TOGGLE, GnumericCellRendererToggleClass))
+#define GNUMERIC_IS_CELL_RENDERER_TOGGLE(o)		(G_TYPE_CHECK_INSTANCE_TYPE ((o), GNUMERIC_TYPE_CELL_RENDERER_TOGGLE))
+#define GNUMERIC_IS_CELL_RENDERER_TOGGLE_CLASS(k)	(G_TYPE_CHECK_CLASS_TYPE ((k), GNUMERIC_TYPE_CELL_RENDERER_TOGGLE))
+#define GNUMERIC_CELL_RENDERER_TOGGLE_GET_CLASS(o)i	(GTK_CHECK_GET_CLASS ((o), GNUMERIC_TYPE_CELL_RENDERER_TOGGLE, GnumericCellRendererToggleClass))
 
 typedef struct _GnumericCellRendererToggle      GnumericCellRendererToggle;
 typedef struct _GnumericCellRendererToggleClass GnumericCellRendererToggleClass;
